/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * <p>
 *    An expression to describe expressions with binary operators.
 * </p>
 *
 * <p>
 *    The operators supported are limited only by the parser's ability to
 *    recognize the operator as a binary (probably mathematical) expression,
 *    and the availability of a function which matches the operator.
 * </p>
 *
 * @author Chris Schultz
 * @version $Revision: 1.2 $ $Date: 2008-06-20 22:58:48 $
 */
public class BinaryOperatorExpression
    extends Expression
{
    /**
     * The left-hand operand of this binary expression.
     */
    private Expression _left;

    /**
     * The operator operating on the left- and right-hand expressions.
     */
    private String _operator;

    /**
     * The right-hand operand of this binary expression.
     */
    private Expression _right;

    /**
     * The argument expressions for this BinaryOperatorExpression.
     */
    private List _argumentExpressions;

    /**
     * Creates a new BinaryOperatorExpression with the specified sub expressions
     * and operator.
     *
     * @param left The left-hand operand of this binary expresssion.
     * @param operator The operator to be used (i.e. "+", "-", "&&", etc.)
     * @param right The right-hand operand of this binary expresssion.
     *
     * @throws IllegalArgumentException If either of the sub-expressions
     *                                  are <code>null</code>, or the operator
     *                                  is not recognized.
     */
    public BinaryOperatorExpression(Expression left,
                          String operator,
                          Expression right)
        throws EvaluationException
    {
        if(null == left)
            throw new IllegalArgumentException("Left-hand side of a BinaryOperatorExpression must not be null.");
        if(null == right)
            throw new IllegalArgumentException("Right-hand side of a BinaryOperatorExpression must not be null.");
        if(null == operator)
            throw new IllegalArgumentException("Operator must not be null.");

        _left = left;
        _operator = operator;
        _right = right;

        left.setParent(this);
        right.setParent(this);

        _argumentExpressions = new ArrayList(2);
        _argumentExpressions.add(left);
        _argumentExpressions.add(right);

        _argumentExpressions = Collections.unmodifiableList(_argumentExpressions);
    }

    /**
     * Gets the expression which is the left-hand operand of this binary
     * operator.
     *
     * @return The expression which is the left-hand operand of this binary
     *         operator.
     */
    public Expression getLeftExpression()
    {
        return _left;
    }

    /**
     * <p>
     *    Gets the operator for this BinaryOperatorExpression.
     * </p>
     *
     * <p>
     *    Note that the operator is nothing magic: it's just the name
     *    of a function that will be looked-up in the EvaluationContext
     *    to determine which function to invoke on the two arguments.
     * </p>
     *
     * @return The operator for this BinaryOperatorExpression.
     */
    public String getOperator()
    {
        return _operator;
    }

    /**
     * Gets the expression which is the right-hand operand of this binary
     * operator.
     *
     * @return The expression which is the right-hand operand of this binary
     *         operator.
     */
    public Expression getRightExpression()
    {
        return _right;
    }

    /**
     * Returns a List containing both sides of this binary BinaryOperatorExpression.
     *
     * @return A List containing both sides of this binary BinaryOperatorExpression.
     */
    public List getSubExpressions()
    {
        ArrayList subs = new ArrayList(3);

        subs.addAll(_left.getSubExpressions());
        subs.add(this);
        subs.addAll(_right.getSubExpressions());

        return subs;
    }

    public void acceptVisitor(ExpressionVisitor visitor)
    {
        visitor.visitBinaryOperatorExpression(this);
    }

    /**
     * Evaluates this Expression by returning the appropriate combination
     * of the left and right-hand sub expressions with the operator.
     *
     * Mathmatical operators return the same type as their inputs (i.e.
     * {@link Integer}s). Logical and relational operators return objects of
     * type {@link Boolean}.
     *
     * @param ec The EvaluationContext in which this expression should
     *           be evaluated.
     *
     * @return The value of this BinaryOperatorExpression.
     *
     * @throws EvaluationException If there is a problem performing the
     *                             mathematical operation (which is
     *                             a function).
     */
    public Object evaluate(EvaluationContext ec)
        throws EvaluationException
    {
        Object func = ec.get(_operator);

        if(null == func)
            throw new EvaluationException("Can't find function implementation for " + _operator);
        else if(func instanceof Function)
            return ((Function)func).call(ec, _operator, _argumentExpressions);
        else
            throw new EvaluationException("Function reference is not a function. It is " + func.getClass().getName() + ", instead.");
    }

    /**
     * Returns an infix String representation of this BinaryOperatorExpression.
     *
     * @return An infix String representation of this BinaryOperatorExpression.
     */
    public String toString()
    {
        return "( " + _left + " "
            + _operator
            + " " + _right + " )";
    }

    /**
     * Returns <code>true</code> if the specified object is equal to
     * this one.
     *
     * @param o The object to test for equality.
     *
     * @return <code>true</code> if the specified object is equal to
     *         this one, <code>false</code> otherwise.
     */
    public boolean equals(Object o)
    {
        if(this == o)
            return true;

        if((null == o)
           || (!this.getClass().equals(o.getClass())))
            return false;

        BinaryOperatorExpression that = (BinaryOperatorExpression)o;

        return ((this._operator.equals(that._operator))
                && this._left.equals(that._left)
                && this._right.equals(that._right));
    }

    public int hashCode()
    {
        return _left.hashCode() ^ _right.hashCode();
    }
}
