/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator;

import java.util.List;

/**
 * An evaluatable Expression.
 *
 * To obtain an Expression, you should use the ExpressionParser class:
 *
 * <pre>
 * Expression exp = ExpressionParser.parseExpression("1 + 2 + 3 * 5");
 * </pre>
 *
 * This expression can later be evaluated:
 *
 * <pre>
 * EvaluationContext ctx = ...;
 * Object result = exp.evaluate(ctx);
 * </pre>
 *
 * The expression in the example will return an Integer result.
 *
 * @author Chris Schultz
 * @version $Revision: 1.3 $ $Date: 2008-06-20 22:58:48 $
 */
public abstract class Expression
{
    /**
     * Evaluates this expression and returns the result.
     *
     * This method is <i>not</i> threadsafe, but is idempotent
     * (repeatable with the same result).
     *
     * @param ec The context in which this expression should
     *           be evaluated.
     *
     * @return The result of evaluating this Expression. This may
     *         by an object of any type, including <code>null</code>.
     *
     * @throws EvaluationException If there is an error evaluating
     *                             this Expression.
     */
    public abstract Object evaluate(EvaluationContext ec)
	throws EvaluationException;

    /**
     * Gets a flattened, depth-first traversal of the expressions in this
     * Expression (including this Expression).
     *
     * @return A List of sub-expressions for this Expression.
     */
    public abstract List getSubExpressions();

    // TODO: This replaces getSubExpressions
    /**
     * Accepts an ExpressionVisitor.
     */
    public abstract void acceptVisitor(ExpressionVisitor visitor);

    protected void setParent(Expression parent)
	throws EvaluationException
    {
	if(null == parent)
	    throw new IllegalArgumentException("Parent must not be null.");

	if(null != _parent)
	    throw new EvaluationException("Expression already has a parent.");

	_parent = parent;
    }

    private Expression _parent;
    public Expression getParent()
    {
	return _parent;
    }
}
