/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.function;

import java.util.Collection;
import java.util.Arrays;

import net.christopherschultz.evaluator.EvaluationContext;
import net.christopherschultz.evaluator.EvaluationException;
import net.christopherschultz.evaluator.StandardFunction;
import net.christopherschultz.evaluator.util.PrimitiveArrayWrapper;

/**
 * <p>
 *    Returns <code>true</code> if the array or collection in the first
 *    argument contains all remaining arguments, in any order.
 * </p>
 *
 * <p>
 *    Usage: containsAll(a:array or list, element1, element2, ... elementN)
 * </p>
 *
 * <p>
 *    This function respects the value of
 *    {@link MathFunction#NULL_RETURNS_NULL}; if the array or list argument
 *    is <code>null</code>, the behavior is defined by the current value
 *    of MathExpression.NULL_RETURNS_NULL. If the property is set to
 *    <code>true</code>, then a <code>null</code> is returned. If the property
 *    is set to <code>false</code>, then <code>false</code> is returned
 *    (since null cannot contain anything).
 * </p>
 *
 * @author Chris Schultz
 * @version $Revision: 1.4 $ $Date: 2008-06-20 22:58:48 $
 */
public class ContainsAllFunction
    extends StandardFunction
{
    public Object call(EvaluationContext ec,
                       String functionName,
                       Object[] args)
        throws EvaluationException
    {
        if(null == args || 2 > args.length)
            throw new EvaluationException("Expected 2 or more arguments, got " + (null == args ? "0" : String.valueOf(args.length)));

        Object haystack = args[0];
        Collection c;
        if(null == haystack)
        {
            if(Boolean.FALSE.equals(ec.getProperty(MathFunction.NULL_RETURNS_NULL)))
                return Boolean.FALSE; // Cannot containAll
            else
                return null; // Default is NULL_RETURNS_NULL => true
        }
        if(haystack instanceof Collection)
        {
            c = (Collection)haystack;
        }
        else if(haystack.getClass().isArray())
        {
            if(haystack.getClass().getComponentType().isPrimitive())
            {
                c = new PrimitiveArrayWrapper(haystack);
            }
            else
            {
                c = java.util.Arrays.asList((Object[])haystack);
            }
        }
        else
            throw new EvaluationException("First argument must be an array or list (was " + haystack.getClass().getName() + " instead)");

        for(int i=1; i<args.length; ++i)
            if(!c.contains(args[i]))
                return Boolean.FALSE;

        // Must have found everything
        return Boolean.TRUE;
    }
}
