/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.function;

import java.util.Collection;
import java.util.Arrays;

import net.christopherschultz.evaluator.EvaluationContext;
import net.christopherschultz.evaluator.EvaluationException;
import net.christopherschultz.evaluator.StandardFunction;

/**
 * <p>
 *    Returns the length of the specified array or collection.
 * </p>
 *
 * <p>
 *    Usage: length(a:array or list)
 * </p>
 *
 * <p>
 *    This function respects the value of
 *    {@link MathFunction#NULL_RETURNS_NULL}; if the array or list argument
 *    is <code>null</code>, the behavior is defined by the current value
 *    of MathExpression.NULL_RETURNS_NULL. If the property is set to
 *    <code>true</code>, then a <code>null</code> is returned. If the property
 *    is set to <code>false</code>, then zero is returned
 *    (since null contains no elements).
 * </p>
 *
 * @author Chris Schultz
 * @version $Revision: 1.2 $ $Date: 2008-06-20 22:58:48 $
 */
public class LengthFunction
    extends StandardFunction
{
    private static final Integer ZERO = new Integer(0);

    public Object call(EvaluationContext ec,
                       String functionName,
                       Object[] args)
        throws EvaluationException
    {
        if(null == args || 1 != args.length)
            throw new EvaluationException("Expected 1 argument, got " + (null == args ? "0" : String.valueOf(args.length)));

        Object haystack = args[0];

        if(null == haystack)
        {
            if(Boolean.FALSE.equals(ec.getProperty(MathFunction.NULL_RETURNS_NULL)))
                return ZERO; // Contains zero elements
            else
                return null; // Default is NULL_RETURNS_NULL => true
        }
        if(haystack instanceof Collection)
        {
            return new Integer(((Collection)haystack).size());
        }
        else if(haystack.getClass().isArray())
        {
            return new Integer(java.lang.reflect.Array.getLength(haystack));
        }
        else
            throw new EvaluationException("First argument must be an array or list (was " + haystack.getClass().getName() + " instead)");
    }
}
