/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.function;

import java.util.Iterator;
import java.util.List;

import net.christopherschultz.evaluator.EvaluationContext;
import net.christopherschultz.evaluator.EvaluationException;
import net.christopherschultz.evaluator.Function;
import net.christopherschultz.evaluator.StandardFunction;

/**
 * <p>
 *    Implements a non-short-circuit boolean AND operation for an arbitrary number
 *    of arguments.
 * </p>
 *
 * <p>
 *    In a short-circuit boolean operation, arguments are evaluated
 *    in a left-to-right fashion, stopping at the fist argument that
 *    fails to meet the criteria. For example, an AND operation would
 *    immediately return FALSE at the first FALSE argument that was
 *    encountered. Likewise, an OR operation would immediately return
 *    TRUE at the first TRUE argument that was encountered.
 * </p>
 *
 * <p>
 *    This function will return <code>null</code> if any <code>null</code>
 *    argument is present, regardless of the others. That means that this
 *    function will only return a definite value (<code>true</code> or
 *    <code>false</code>) if all arguments have values.
 * </p>
 *
 * @author Chris Schultz
 * @version $Revision: 1.2 $ $Date: 2008-06-20 22:58:48 $
 */
public class NonShortCircuitAnd
    implements Function
{
    /**
     * Creates a new NonShortCircuitAnd object.
     */
    public NonShortCircuitAnd()
    {
        super();
    }

    public Object call(EvaluationContext ec,
		       String functionName,
		       List argumentExpressions)
	throws EvaluationException
    {
	if(null == argumentExpressions || 0 == argumentExpressions.size())
	    throw new EvaluationException("Inappropriate number of objects: " + (null == argumentExpressions ? "0" : String.valueOf(argumentExpressions.size())));

	boolean value = true;
	for(Iterator i=argumentExpressions.iterator(); i.hasNext(); )
	{
	    Object result = StandardFunction.getExpressionValue(ec, i.next());

	    if(null == result)
	    {
		return null;
	    }
	    else if(result instanceof Boolean)
	    {
		value &= ((Boolean)result).booleanValue();
	    }
	    else
		throw new EvaluationException("Encountered non-boolean argument: "
					      + result);
	}

	return value ? Boolean.TRUE : Boolean.FALSE;
    }

    public String toString()
    {
	return "NonShortCircuitAnd";
    }
}
