/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.function;

import java.util.Iterator;
import java.util.List;

import net.christopherschultz.evaluator.EvaluationContext;
import net.christopherschultz.evaluator.EvaluationException;
import net.christopherschultz.evaluator.Function;
import net.christopherschultz.evaluator.StandardFunction;

/**
 * <p>
 *    Implements a short-circuit boolean AND operation for an arbitrary number
 *    of arguments.
 * </p>
 *
 * <p>
 *    In a short-circuit boolean operation, arguments are evaluated
 *    in a left-to-right fashion, stopping at the fist argument that
 *    fails to meet the criteria. For example, an AND operation would
 *    immediately return FALSE at the first FALSE argument that was
 *    encountered. Likewise, an OR operation would immediately return
 *    TRUE at the first TRUE argument that was encountered.
 * </p>
 *
 * <p>
 *    If a <code>null</code> is encountered before the operation can
 *    determine a value (possibly by short-circuiting), then <code>null</code>
 *    is returned.
 * </p>
 *
 * @author Chris Schultz
 * @version $Revision: 1.5 $ $Date: 2008-06-20 22:58:48 $
 */
public class ShortCircuitAnd
    implements Function
{
    /**
     * Creates a new ShortCircuitAnd object.
     */
    public ShortCircuitAnd()
    {
        super();
    }

    public Object call(EvaluationContext ec,
		       String functionName,
		       List argumentExpressions)
	throws EvaluationException
    {
	if(null == argumentExpressions || 0 == argumentExpressions.size())
	    throw new EvaluationException("Inappropriate number of objects: " + (null == argumentExpressions ? "0" : String.valueOf(argumentExpressions.size())));

	boolean sawNull = false;
	for(Iterator i=argumentExpressions.iterator(); i.hasNext(); )
	{
	    Object result = StandardFunction.getExpressionValue(ec, i.next());

	    if(null == result)
	    {
		sawNull = true;
	    }
	    else if(result instanceof Boolean)
	    {
		// Any FALSE is a deal-breaker

		if(!((Boolean)result).booleanValue())
		    return Boolean.FALSE;
	    }
	    else
		throw new EvaluationException("Encountered non-boolean argument: " + result);
	}

	if(sawNull)
	    return null;
	else
	    return Boolean.TRUE;
    }

    public String toString()
    {
	return "ShortCircuitAnd";
    }
}
