/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.parser;

import java.io.StringReader;

import net.christopherschultz.evaluator.EvaluationException;
import net.christopherschultz.evaluator.Expression;

/**
 * A parser to convert string expressions into Expression
 * objects for subsequent evaluation.
 *
 * @author Chris Schultz
 * @version $Revision: 1.4 $ $Date: 2009-04-09 16:02:58 $
 *
 * @see Expression
 */
public class ExpressionParser
{
    /**
     * Creates a new ExpressionParser.
     */
    public ExpressionParser()
    {
        super();
    }

    /**
     * Parses the given string expression into a tree of evaluatable
     * Expression objects.
     *
     * @param expression The string expression to parse.
     *
     * @return An Expression object which is the root of an expression
     *         evaluation tree.
     *
     * @throws ParseException If the expression was not parsable.
     */
    public Expression parse(String expression)
        throws ParseException
    {
        try
        {
            ExpressionLexerImpl lexer = new ExpressionLexerImpl(new StringReader(expression));
            ExpressionParserImpl parser = new ExpressionParserImpl(lexer);

            return(parser.expression());
        }
        catch (antlr.RecognitionException re)
        {
            throw new ParseException("Invalid expression: " + expression, re);
        }
        catch (antlr.TokenStreamException tse)
        {
            throw new ParseException("Invalid expression: " + expression, tse);
        }
        catch (EvaluationException ee)
        {
            throw new ParseException("Internal expression error??", ee);
        }
    }

    /**
     * A convenience method to create a new ExpressionParser and
     * return an Expression object for that expression.
     *
     * @param expression The string expression to parse.
     *
     * @return An Expression object which is the root of an expression
     *         evaluation tree.
     *
     * @throws ParseException If the expression was not parsable.
     */
    public static Expression parseExpression(String expression)
        throws ParseException
    {
        return new ExpressionParser().parse(expression);
    }
}
