/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.util;

import java.util.HashMap;
import java.util.Map;

import net.christopherschultz.evaluator.EvaluationContext;
import net.christopherschultz.evaluator.Function;

/**
 * <p>
 *    Provides a wrapper around another EvaluationContext.
 * </p>
 *
 * <p>
 *    This is useful when you want to have a shared, global set of
 *    functions and values, but also want to have some functions and
 *    values local to one EvaluationContext.
 * </p>
 *
 * <p>
 *    This class provides read-only access to the parent EvaluationContext,
 *    and searches the local environment before searching the parent.
 * </p>
 *
 * @author Chris Schultz
 * @version $Revision: 1.6 $ $Date: 2008-06-20 22:58:48 $
 */
public class ChainedEvaluationContext
    extends DefaultEvaluationContext
{
    /**
     * The parent EvaluationContext of this ChainedEvaluationContext.
     */
    private EvaluationContext _parent;

    /**
     * Creates a new ChainedEvaluationContext.
     *
     * @param parent The parent EvaluationContext.
     */
    public ChainedEvaluationContext(EvaluationContext parent)
    {
        if(null == parent)
            throw new IllegalArgumentException("Parent must not be null");

        _parent = parent;
    }

    /**
     * Gets the value of the specified identifier.
     *
     * @param identifier The identifier for the value.
     *
     * @return The value of the specified value.
     */
    public Object get(String identifier)
    {
        Object value = super.get(identifier);

        if(null == value)
            value = _parent.get(identifier);

        return value;
    }

    public Map getAll()
    {
        HashMap map = new HashMap();

        map.putAll(_parent.getAll());
        map.putAll(super.getAll());

        return map;
    }
}
