/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.util;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.ResourceBundle;

import net.christopherschultz.evaluator.EvaluationContext;
import net.christopherschultz.evaluator.Function;

/**
 * <p>
 *    Contains important information about the current expression
 *    evaluation.
 * </p>
 *
 * <p>
 *    Using the {@link #loadStandard} method, you can have this
 *    EvaluationContext load "standard" functions and constants
 *    into the context from the file <code>EvaluationContext.properties</code>.
 *    Using the {@link #load(ResourceBundle)} or {@link #load(Map)} methods,
 *    you can load those constants and function definitions from arbitrary
 *    sources.
 * </p>
 *
 * <p>
 *    I also might need some flags, like, 'should a NULL return object result
 *    in the expression evaluation terminating, returning nulls until the final
 *    return?'. That would be useful for when you want to vail out in the
 *    case that one of your functions returns null denoting that the expression
 *    <i>cannot</i> currently be evaluated.
 * </p>
 *
 * @author Chris Schultz
 * @version $Revision: 1.6 $ $Date: 2008-06-20 22:58:48 $
 */
public class DefaultEvaluationContext
    implements EvaluationContext
{
    /**
     * The registry of values for this EvaluationContext.
     */
    private Map _values;

    /**
     * The properties of this EvaluationContext.
     */
    private Map _properties;

    /**
     * <p>
     *    Creates a new EvaluationContext.
     * </p>
     *
     * <p>
     *    Note that no functions, constants, etc. will be defined
     *    in this EvaluationContext. If you want to load all the
     *    "standard" functions and constants, as defined by
     *    EvaluationContext.properties, then call the
     *    {@link #loadStandard} method. If you want to load
     *    your own functions, constants, and properties, then
     *    use {@link #load(ResourceBundle)} or {@link #load(Map)}.
     * </p>
     */
    public DefaultEvaluationContext()
    {
        _values = new HashMap();
        _properties = new HashMap();
    }

    /**
     * Gets the value of the specified identifier.
     *
     * @param identifier The identifier for the value.
     *
     * @return The value of the specified value.
     */
    public Object get(String identifier)
    {
        return _values.get(identifier);
    }

    /**
     * Returns a Map of all bound identifiers and their values.
     * The Map is not modifiable.
     *
     * @return A Map of all identifiers as keys and their values
     *         as the values of the Map.
     */
    public Map getAll()
    {
        return Collections.unmodifiableMap(_values);
    }

    /**
     * Sets the value of the specified identifer to the given value.
     *
     * @param identifier The identifier for the value.
     * @param value The value for the identifier.
     */
    public Object set(String identifier, Object value)
    {
        Object oldValue = _values.put(identifier, value);

        // Did the value change?
        if(null != oldValue && !oldValue.equals(value))
        {
            System.err.println("Replacing value '" + identifier
                               + "'. Value was " + oldValue
                               + ", is now " + value);
        }

        return value;
    }

    public Object remove(String identifier)
    {
        return _values.remove(identifier);
    }

    /**
     * Sets the specified property to the given value.
     *
     * @param key The name of the property.
     * @param value The value of the property.
     */
    public void setProperty(String key, Object value)
    {
        _properties.put(key, value);
    }

    /**
     * Gets the specified property.
     *
     * @param key The name of the property.
     *
     * @return The value of the specified property.
     */
    public Object getProperty(String key)
    {
        return _properties.get(key);
    }

    /**
     * Removes the specified property.
     *
     * @param key The name of the property to remove.
     */
    public void removeProperty(Object key)
    {
        _properties.remove(key);
    }

    public EvaluationContext loadStandard()
    {
        load(ResourceBundle.getBundle(EvaluationContext.class.getName()));

        return this;
    }

    public void load(ResourceBundle bundle)
    {
        ContextUtil.load(bundle, this);
    }
    
    public void load(Map props)
    {
        ContextUtil.load(props, this);
    }
}
