/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator;

import junit.framework.Assert;
import junit.framework.Test;
import junit.framework.TestSuite;

import org.jmock.MockObjectTestCase;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * UnitTest for expressions and evaluating them.<p>
 *
 * @author Chris Schultz
 * @version $Revision: 1.3 $ $Date: 2008-06-20 22:58:48 $
 */
public class UnitTest
    extends MockObjectTestCase
{
    protected static IdentifierExpression id(String s)
    {
        return new IdentifierExpression(s);
    }
    protected static ConstantExpression constant(int i)
    {
        return new ConstantExpression(new Integer(i));
    }

    protected static ConstantExpression constant(boolean b)
    {
        return new ConstantExpression(b ? Boolean.TRUE : Boolean.FALSE);
    }

    protected static ConstantExpression constant(Object o)
    {
        return new ConstantExpression(o);
    }


    //
    // ===================
    // Stupid-simple tests
    //====================
    //

    public void assertNotEquals(String message,
                                Object expected,
                                Object actual)
    {
        Assert.assertTrue(message,
                          !(expected.equals(actual)));
    }

    public void testObjectEquals()
        throws Exception
    {
        // Test booleans
        Assert.assertEquals("ConstantExpression.equals fails",
                            constant(true),
                            constant(true));

        Assert.assertEquals("ConstantExpression.equals fails",
                            constant(false),
                            constant(false));

        assertNotEquals("ConstantExpression.equals fails",
                        constant(true),
                        constant(false));

        assertNotEquals("ConstantExpression.equals fails",
                        constant(false),
                        constant(true));

        // Test ints
        Assert.assertEquals("ConstantExpression.equals fails",
                            constant(24),
                            constant(24));

        assertNotEquals("ConstantExpression.equals fails",
                        constant(24),
                        constant(207));

        // Test Strings
        Assert.assertEquals("ConstantExpression.equals fails",
                            constant("test"),
                            constant("test"));

        assertNotEquals("ConstantExpression.equals fails",
                        constant("test"),
                        constant("!test"));

        // Test negations
        Assert.assertEquals("NegatedExpression.equals fails",
                            new NegatedExpression(constant(false)),
                            new NegatedExpression(constant(false)));

        // Test function calls
        ArrayList arguments = new ArrayList();
        ArrayList arguments2 = new ArrayList();
        arguments2.add(constant(2));

        Assert.assertEquals("FunctionCallExpression.equals fails",
                            new FunctionCallExpression(id("count"), arguments),
                            new FunctionCallExpression(id("count"), arguments));
						       
        assertNotEquals("FunctionCallExpression.equals fails",
                        new FunctionCallExpression(id("count"), arguments),
                        new FunctionCallExpression(id("count"), arguments2));

        assertNotEquals("FunctionCallExpression.equals fails",
                        new FunctionCallExpression(id("count"), arguments),
                        new FunctionCallExpression(id("age"), arguments));

        // Test BinaryOperatorExpression
        Assert.assertEquals("BinaryOperatorExpression.equals fails",
                            new BinaryOperatorExpression(constant(25),
                                                         "+",
                                                         constant(17)),
                            new BinaryOperatorExpression(constant(25),
                                                         "+",
                                                         constant(17)));

        assertNotEquals("BinaryOperatorExpression.equals fails",
                        new BinaryOperatorExpression(constant(25),
                                                     "+",
                                                     constant(17)),
                        new BinaryOperatorExpression(constant(25),
                                                     "-",
                                                     constant(17)));
        assertNotEquals("BinaryOperatorExpression.equals fails",
                        new BinaryOperatorExpression(constant(25),
                                                     "+",
                                                     constant(17)),
                        new BinaryOperatorExpression(constant(25),
                                                     "+",
                                                     constant(180)));
    }

    class ExpressionStringGenerator
        implements ExpressionVisitor
    {
        String expr = new String();

        public void visitBinaryOperatorExpression(BinaryOperatorExpression exp)
        {
            expr += "( ";
            exp.getLeftExpression().acceptVisitor(this);
            expr += " ";
            expr += exp.getOperator();
            expr += " ";
            exp.getRightExpression().acceptVisitor(this);
            expr += " )";
        }
        public void visitFunctionCallExpression(FunctionCallExpression exp)
        {
            exp.getFunctionExpression().acceptVisitor(this);
            expr += "(";
            for(java.util.Iterator i=exp.getArgumentExpressions().iterator();
                i.hasNext(); )
            {
                ((Expression)i.next()).acceptVisitor(this);
                if(i.hasNext())
                    expr += ", ";
            }
            expr += ")";
        }
        public void visitArrayReferenceExpression(ArrayReferenceExpression exp)
        {
            exp.getArrayExpression().acceptVisitor(this);
            expr += "[";
            exp.getIndexExpression().acceptVisitor(this);
            expr += "]";
        }
        public void visitNegatedExpression(NegatedExpression exp)
        {
            expr += "!(";
            exp.getSubExpression().acceptVisitor(this);
            expr += ")";
        }
        public void visitConstantExpression(ConstantExpression exp)
        {
            expr += exp.getValue();
        }
        public void visitIdentifierExpression(IdentifierExpression exp)
        {
            expr += exp.getValue();
        }
        public void visitUnaryMinusExpression(UnaryMinusExpression exp)
        {
            expr += "-(";
            exp.getSubExpression().acceptVisitor(this);
            expr += ")";
        }
    }

    class FunctionFinder
        implements ExpressionVisitor
    {
        ArrayList functions = new ArrayList();
        
        public void visitBinaryOperatorExpression(BinaryOperatorExpression exp)
        {
            exp.getLeftExpression().acceptVisitor(this);
            exp.getRightExpression().acceptVisitor(this);
        }
        public void visitFunctionCallExpression(FunctionCallExpression exp)
        {
            functions.add(exp);

            for(java.util.Iterator i=exp.getArgumentExpressions().iterator();
                i.hasNext(); )
                ((Expression)i.next()).acceptVisitor(this);
        }
        public void visitArrayReferenceExpression(ArrayReferenceExpression exp)
        {
            exp.getArrayExpression().acceptVisitor(this);
            exp.getIndexExpression().acceptVisitor(this);
        }
        public void visitNegatedExpression(NegatedExpression exp)
        {
            exp.getSubExpression().acceptVisitor(this);
        }
        public void visitConstantExpression(ConstantExpression exp)
        {
            /* Do nothing */
        }
        public void visitIdentifierExpression(IdentifierExpression exp)
        {
            /* Do nothing */
        }
        
        public void visitUnaryMinusExpression(UnaryMinusExpression exp)
        {
            exp.getSubExpression().acceptVisitor(this);
        }
	}

    public void testExpressionStringGenerator()
        throws Exception
    {
        Expression exp = net.christopherschultz.evaluator.parser.ExpressionParser.parseExpression("false || 4 < count(true, 5, 6, 7)");

        ExpressionStringGenerator esg = new ExpressionStringGenerator();
        exp.acceptVisitor(esg);

        Assert.assertEquals("Expression string is incorrect",
                            "( false || ( 4 < count(true, 5, 6, 7) ) )", esg.expr);
    }

    public void testFunctionFinder()
        throws Exception
    {
        Expression exp = net.christopherschultz.evaluator.parser.ExpressionParser.parseExpression("false || 4 < count(true, 5, 6, 7)");

        FunctionFinder ff = new FunctionFinder();
        exp.acceptVisitor(ff);

        ArrayList functions = new ArrayList();
        functions.add(((BinaryOperatorExpression)
                       ((BinaryOperatorExpression)exp).getRightExpression()
                       ).getRightExpression());

        Assert.assertEquals("Didn't find the expected function",
                            functions, ff.functions);
    }

    public void testParentSetting()
        throws Exception
    {
        UnaryMinusExpression ume = new UnaryMinusExpression(new ConstantExpression(new Integer(1)));

        Assert.assertEquals("UnaryMinusExpression doesn't properly set its children's parent",
                            ume, ume.getSubExpression().getParent());

        NegatedExpression ne = new NegatedExpression(new ConstantExpression(new Integer(1)));

        Assert.assertEquals("NegatedExpression doesn't properly set its children's parent",
                            ne, ne.getSubExpression().getParent());

        BinaryOperatorExpression me =  new BinaryOperatorExpression(new ConstantExpression(new Integer(1)),
                                                                    "=",
                                                                    new ConstantExpression(new Integer(2)));

        Assert.assertEquals("BinaryOperatorExpression doesn't properly set its children's parent",
                            me, me.getLeftExpression().getParent());

        ArrayList args = new ArrayList();
        args.add(new ConstantExpression(new Integer(1)));
        args.add(new ConstantExpression(new Integer(2)));
        args.add(new ConstantExpression(new Integer(3)));

        FunctionCallExpression fce =  new FunctionCallExpression(id("f"),
                                                                 args);

        for(Iterator i=args.iterator(); i.hasNext(); )
        {
            Expression arg = (Expression)i.next();

            Assert.assertEquals("FunctionCallExpression doesn't properly set its childrens' parents",
                                fce, arg.getParent());
        }
    }

    /**
     * Returns a Test which will execute all the test methods found
     * in this class.<p>
     *
     * @return A Test for JUnit to run.
     */
    public static Test suite()
    {
        TestSuite ts = new TestSuite(UnitTest.class);

        return ts;
    }

    /**
     * Creates a new UnitTest to execute the specified test.<p>
     *
     * @param name The name of the test method to run.
     */
    public UnitTest(String name)
    {
        super(name);
    }
}
