/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.function;

import junit.framework.Assert;
import junit.framework.TestCase;

import net.christopherschultz.evaluator.util.DefaultEvaluationContext;
import net.christopherschultz.evaluator.ConstantExpression;
import net.christopherschultz.evaluator.EvaluationContext;
import net.christopherschultz.evaluator.EvaluationException;
import net.christopherschultz.evaluator.Expression;
import net.christopherschultz.evaluator.IdentifierExpression;
import net.christopherschultz.evaluator.BinaryOperatorExpression;

/**
 * UnitTest for BinaryOperator functions.
 *
 * @author Chris Schultz
 * @version $Revision: 1.3 $ $Date: 2008-06-20 22:58:48 $
 */
public class BinaryOperatorTests
    extends TestCase
{
    EvaluationContext ec;

    public void setUp()
    {
        ec = new DefaultEvaluationContext();

        ec.set("&&", new BinaryOperator.And());
        ec.set("||", new BinaryOperator.Or());

        ec.set("+", new BinaryOperator.Add());
        ec.set("-", new BinaryOperator.Subtract());
        ec.set("*", new BinaryOperator.Multiply());
        ec.set("/", new BinaryOperator.Divide());
        ec.set("%", new BinaryOperator.Modulus());

        ec.set("=", new BinaryOperator.Equal());
        ec.set("!=", new BinaryOperator.NotEqual());
        ec.set("<", new BinaryOperator.Less());
        ec.set(">", new BinaryOperator.Greater());
        ec.set("<=", new BinaryOperator.LessEqual());
        ec.set(">=", new BinaryOperator.GreaterEqual());
    }

    private Expression nullExp() { return new IdentifierExpression("null");}

    public void testNullsAddition()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(nullExp(),
                                                      "+",
                                                      nullExp());

        Assert.assertNull("Adding nulls does not return null",
                          exp.evaluate(ec));
    }

    public void testLeftNullAddition()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(nullExp(),
                                                      "+",
                                                      new ConstantExpression(new Integer(1)));

        Assert.assertNull("Adding left null does not return null",
                          exp.evaluate(ec));
    }

    public void testRightNullAddition()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(1)),
                                                      "+",
                                                      nullExp());

        Assert.assertNull("Adding right null does not return null",
                          exp.evaluate(ec));
    }

    public void testIntegerAddition()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                      "+",
                                                      new ConstantExpression(new Integer(17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple addition results in null", result);
        Assert.assertEquals("Integer addition does not return an integer",
                            Integer.class, result.getClass());
        Assert.assertEquals("Addition doesn't add properly",
                            42+17, ((Integer)result).intValue());
    }

    public void testLongAddition()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Long(42)),
                                                      "+",
                                                      new ConstantExpression(new Long(17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple addition results in null", result);
        Assert.assertEquals("Long addition does not return an integer",
                            Long.class, result.getClass());
        Assert.assertEquals("Addition doesn't add properly",
                            (42+17), ((Long)result).longValue());
    }

    public void testFloatAddition()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Float(42.1)),
                                                      "+",
                                                      new ConstantExpression(new Float(17.1)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple addition results in null", result);
        Assert.assertEquals("Float addition does not return a float",
                            Float.class, result.getClass());
        Assert.assertEquals("Addition doesn't add properly",
                            (float)(42.1+17.1), ((Float)result).floatValue(),
                            0.01f);
    }

    public void testDoubleAddition()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Double(42.1)),
                                                      "+",
                                                      new ConstantExpression(new Double(17.1)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple addition results in null", result);
        Assert.assertEquals("Double addition does not return a double",
                            Double.class, result.getClass());
        Assert.assertEquals("Addition doesn't add properly",
                            42.1+17.1, ((Double)result).doubleValue(), 0.0);
    }

    public void testMixedAdditionDoubleInt()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Double(42.1)),
                                                      "+",
                                                      new ConstantExpression(new Integer(17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple addition results in null", result);
        Assert.assertEquals("Mixed (double,int) addition does not return a double",
                            Double.class, result.getClass());
        Assert.assertEquals("Addition doesn't add properly",
                            42.1+17, ((Double)result).doubleValue(), 0.0);
    }


    public void testMixedAdditionDoubleLong()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Double(42.1)),
                                                      "+",
                                                      new ConstantExpression(new Long(Long.MAX_VALUE)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple addition results in null", result);
        Assert.assertEquals("Mixed (double,long) addition does not return a double",
                            Double.class, result.getClass());
        Assert.assertEquals("Addition doesn't add properly",
                            42.1+Long.MAX_VALUE, ((Double)result).doubleValue(), 0.0);
    }

    public void testMixedAdditionDoubleFloat()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Double(42.1)),
                                                      "+",
                                                      new ConstantExpression(new Float(17.1)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple addition results in null", result);
        Assert.assertEquals("Mixed (double,float) addition does not return a double",
                            Double.class, result.getClass());
        Assert.assertEquals("Addition doesn't add properly",
                            42.1+17.1f, ((Double)result).doubleValue(), 0.0);
    }

    public void testMixedAdditionFloatInt()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Float(42.1)),
                                                      "+",
                                                      new ConstantExpression(new Integer(17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple addition results in null", result);
        Assert.assertEquals("Mixed (float,int) addition does not return a float",
                            Float.class, result.getClass());
        Assert.assertEquals("Addition doesn't add properly",
                            42.1+17, ((Float)result).floatValue(), 0.01);
    }


    public void testMixedAdditionFloatLong()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Float(42.1)),
                                                      "+",
                                                      new ConstantExpression(new Long(Long.MAX_VALUE)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple addition results in null", result);
        Assert.assertEquals("Mixed (float,long) addition does not return a float",
                            Float.class, result.getClass());
        Assert.assertEquals("Addition doesn't add properly",
                            42.1+Long.MAX_VALUE, ((Float)result).floatValue(), 0.0);
    }

    public void testMixedAdditionIntegerByte()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                      "+",
                                                      new ConstantExpression(new Byte((byte)17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple addition results in null", result);
        Assert.assertEquals("Mixed (int,byte) addition does not return an int",
                            Integer.class, result.getClass());
        Assert.assertEquals("Addition doesn't add properly",
                            42+17, ((Integer)result).intValue());
    }

    // Subtraction

    public void testIntegerSubtraction()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                      "-",
                                                      new ConstantExpression(new Integer(17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple subtraction results in null", result);
        Assert.assertEquals("Integer subtraction does not return an integer",
                            Integer.class, result.getClass());
        Assert.assertEquals("Subtraction doesn't subtract properly",
                            42-17, ((Integer)result).intValue());
    }

    public void testLongSubtraction()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Long(42)),
                                                      "-",
                                                      new ConstantExpression(new Long(17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple subtraction results in null", result);
        Assert.assertEquals("Long subtraction does not return an integer",
                            Long.class, result.getClass());
        Assert.assertEquals("Subtraction doesn't subtract properly",
                            (42-17), ((Long)result).longValue());
    }

    public void testFloatSubtraction()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Float(42.1)),
                                                      "-",
                                                      new ConstantExpression(new Float(17.1)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple subtraction results in null", result);
        Assert.assertEquals("Float subtraction does not return a float",
                            Float.class, result.getClass());
        Assert.assertEquals("Subtraction doesn't subtract properly",
                            (float)(42.1-17.1), ((Float)result).floatValue(),
                            0.01f);
    }

    public void testDoubleSubtraction()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Double(42.1)),
                                                      "-",
                                                      new ConstantExpression(new Double(17.1)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple subtraction results in null", result);
        Assert.assertEquals("Double subtraction does not return a double",
                            Double.class, result.getClass());
        Assert.assertEquals("Subtraction doesn't subtract properly",
                            42.1-17.1, ((Double)result).doubleValue(), 0.0);
    }

    public void testMixedSubtractionDoubleInt()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Double(42.1)),
                                                      "-",
                                                      new ConstantExpression(new Integer(17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple subtraction results in null", result);
        Assert.assertEquals("Mixed (double,int) subtraction does not return a double",
                            Double.class, result.getClass());
        Assert.assertEquals("Subtraction doesn't subtract properly",
                            42.1-17, ((Double)result).doubleValue(), 0.0);
    }


    public void testMixedSubtractionDoubleLong()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Double(42.1)),
                                                      "-",
                                                      new ConstantExpression(new Long(Long.MAX_VALUE)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple subtraction results in null", result);
        Assert.assertEquals("Mixed (double,long) subtraction does not return a double",
                            Double.class, result.getClass());
        Assert.assertEquals("Subtraction doesn't subtract properly",
                            42.1-Long.MAX_VALUE, ((Double)result).doubleValue(), 0.0);
    }

    public void testMixedSubtractionDoubleFloat()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Double(42.1)),
                                                      "-",
                                                      new ConstantExpression(new Float(17.1)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple subtraction results in null", result);
        Assert.assertEquals("Mixed (double,float) subtraction does not return a double",
                            Double.class, result.getClass());
        Assert.assertEquals("Subtraction doesn't subtract properly",
                            42.1-17.1f, ((Double)result).doubleValue(), 0.0);
    }

    public void testMixedSubtractionFloatInt()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Float(42.1)),
                                                      "-",
                                                      new ConstantExpression(new Integer(17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple subtraction results in null", result);
        Assert.assertEquals("Mixed (float,int) subtraction does not return a float",
                            Float.class, result.getClass());
        Assert.assertEquals("Subtraction doesn't subtract properly",
                            42.1-17, ((Float)result).floatValue(), 0.01);
    }


    public void testMixedSubtractionFloatLong()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Float(42.1)),
                                                      "-",
                                                      new ConstantExpression(new Long(Long.MAX_VALUE)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple subtraction results in null", result);
        Assert.assertEquals("Mixed (float,long) subtraction does not return a float",
                            Float.class, result.getClass());
        Assert.assertEquals("Subtraction doesn't subtract properly",
                            42.1-Long.MAX_VALUE, ((Float)result).floatValue(), 0.0);
    }

    public void testMixedSubtractionIntegerByte()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                      "-",
                                                      new ConstantExpression(new Byte((byte)17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple subtraction results in null", result);
        Assert.assertEquals("Mixed (int,byte) subtraction does not return an int",
                            Integer.class, result.getClass());
        Assert.assertEquals("Subtraction doesn't subtract properly",
                            42-17, ((Integer)result).intValue());
    }

    public void testNullsSubtraction()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(nullExp(),
                                                      "-",
                                                      nullExp());

        Assert.assertNull("Subtracting nulls does not return null",
                          exp.evaluate(ec));
    }

    public void testLeftNullSubtraction()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(nullExp(),
                                                      "-",
                                                      new ConstantExpression(new Integer(1)));

        Assert.assertNull("Subtracting left null does not return null",
                          exp.evaluate(ec));
    }

    public void testRightNullSubtraction()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(1)),
                                                      "-",
                                                      nullExp());

        Assert.assertNull("Subtracting right null does not return null",
                          exp.evaluate(ec));
    }

    //
    // Multiplication
    //

    public void testIntegerMultiplication()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                      "*",
                                                      new ConstantExpression(new Integer(17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple multiplication results in null", result);
        Assert.assertEquals("Integer multiplication does not return an integer",
                            Integer.class, result.getClass());
        Assert.assertEquals("Multiplication doesn't multiply properly",
                            42*17, ((Integer)result).intValue());
    }

    public void testLongMultiplication()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Long(42)),
                                                      "*",
                                                      new ConstantExpression(new Long(17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple multiplication results in null", result);
        Assert.assertEquals("Long multiplication does not return an integer",
                            Long.class, result.getClass());
        Assert.assertEquals("Multiplication doesn't multiply properly",
                            (42*17), ((Long)result).longValue());
    }

    public void testFloatMultiplication()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Float(42.1)),
                                                      "*",
                                                      new ConstantExpression(new Float(17.1)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple multiplication results in null", result);
        Assert.assertEquals("Float multiplication does not return a float",
                            Float.class, result.getClass());
        Assert.assertEquals("Multiplication doesn't multiply properly",
                            (float)(42.1*17.1), ((Float)result).floatValue(),
                            0.01f);
    }

    public void testDoubleMultiplication()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Double(42.1)),
                                                      "*",
                                                      new ConstantExpression(new Double(17.1)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple multiplication results in null", result);
        Assert.assertEquals("Double multiplication does not return a double",
                            Double.class, result.getClass());
        Assert.assertEquals("Multiplication doesn't multiply properly",
                            42.1*17.1, ((Double)result).doubleValue(), 0.0);
    }

    public void testMixedMultiplicationDoubleInt()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Double(42.1)),
                                                      "*",
                                                      new ConstantExpression(new Integer(17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple multiplication results in null", result);
        Assert.assertEquals("Mixed (double,int) multiplication does not return a double",
                            Double.class, result.getClass());
        Assert.assertEquals("Multiplication doesn't multiply properly",
                            42.1*17, ((Double)result).doubleValue(), 0.0);
    }


    public void testMixedMultiplicationDoubleLong()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Double(42.1)),
                                                      "*",
                                                      new ConstantExpression(new Long(Long.MAX_VALUE)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple multiplication results in null", result);
        Assert.assertEquals("Mixed (double,long) multiplication does not return a double",
                            Double.class, result.getClass());
        Assert.assertEquals("Multiplication doesn't multiply properly",
                            42.1*Long.MAX_VALUE, ((Double)result).doubleValue(), 0.0);
    }

    public void testMixedMultiplicationDoubleFloat()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Double(42.1)),
                                                      "*",
                                                      new ConstantExpression(new Float(17.1)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple multiplication results in null", result);
        Assert.assertEquals("Mixed (double,float) multiplication does not return a double",
                            Double.class, result.getClass());
        Assert.assertEquals("Multiplication doesn't multiply properly",
                            42.1*17.1f, ((Double)result).doubleValue(), 0.0);
    }

    public void testMixedMultiplicationFloatInt()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Float(42.1)),
                                                      "*",
                                                      new ConstantExpression(new Integer(17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple multiplication results in null", result);
        Assert.assertEquals("Mixed (float,int) multiplication does not return a float",
                            Float.class, result.getClass());
        Assert.assertEquals("Multiplication doesn't multiply properly",
                            42.1*17, ((Float)result).floatValue(), 0.01);
    }


    public void testMixedMultiplicationFloatLong()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Float(42.1)),
                                                      "*",
                                                      new ConstantExpression(new Long(Long.MAX_VALUE)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple multiplication results in null", result);
        Assert.assertEquals("Mixed (float,long) multiplication does not return a float",
                            Float.class, result.getClass());
        Assert.assertEquals("Multiplication doesn't multiply properly",
                            (float)(42.1*Long.MAX_VALUE),
                            ((Float)result).floatValue(),
                            0.01f);
    }

    public void testMixedMultiplicationIntegerByte()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                      "*",
                                                      new ConstantExpression(new Byte((byte)17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple multiplication results in null", result);
        Assert.assertEquals("Mixed (int,byte) multiplication does not return an int",
                            Integer.class, result.getClass());
        Assert.assertEquals("Multiplication doesn't multiply properly",
                            42*17, ((Integer)result).intValue());
    }

    public void testNullsMultiplication()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(nullExp(),
                                                      "*",
                                                      nullExp());

        Assert.assertNull("Multiplying nulls does not return null",
                          exp.evaluate(ec));
    }

    public void testLeftNullMultiplication()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(nullExp(),
                                                      "*",
                                                      new ConstantExpression(new Integer(1)));

        Assert.assertNull("Multiplying left null does not return null",
                          exp.evaluate(ec));
    }

    public void testRightNullMultiplication()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(1)),
                                                      "*",
                                                      nullExp());

        Assert.assertNull("Multiplying right null does not return null",
                          exp.evaluate(ec));
    }

    //
    // Division
    //

    public void testIntegerDivision()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                      "/",
                                                      new ConstantExpression(new Integer(17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple division results in null", result);
        Assert.assertEquals("Integer division does not return an integer",
                            Integer.class, result.getClass());
        Assert.assertEquals("Division doesn't divide properly",
                            42/17, ((Integer)result).intValue());
    }

    public void testLongDivision()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Long(42)),
                                                      "/",
                                                      new ConstantExpression(new Long(17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple division results in null", result);
        Assert.assertEquals("Long division does not return an integer",
                            Long.class, result.getClass());
        Assert.assertEquals("Division doesn't divide properly",
                            (42/17), ((Long)result).longValue());
    }

    public void testFloatDivision()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Float(42.1)),
                                                      "/",
                                                      new ConstantExpression(new Float(17.1)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple division results in null", result);
        Assert.assertEquals("Float division does not return a float",
                            Float.class, result.getClass());
        Assert.assertEquals("Division doesn't divide properly",
                            (float)(42.1/17.1), ((Float)result).floatValue(),
                            0.01f);
    }

    public void testDoubleDivision()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Double(42.1)),
                                                      "/",
                                                      new ConstantExpression(new Double(17.1)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple division results in null", result);
        Assert.assertEquals("Double division does not return a double",
                            Double.class, result.getClass());
        Assert.assertEquals("Division doesn't divide properly",
                            42.1/17.1, ((Double)result).doubleValue(), 0.0);
    }

    public void testMixedDivisionDoubleInt()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Double(42.1)),
                                                      "/",
                                                      new ConstantExpression(new Integer(17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple division results in null", result);
        Assert.assertEquals("Mixed (double,int) division does not return a double",
                            Double.class, result.getClass());
        Assert.assertEquals("Division doesn't divide properly",
                            42.1/17, ((Double)result).doubleValue(), 0.0);
    }


    public void testMixedDivisionDoubleLong()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Double(42.1)),
                                                      "/",
                                                      new ConstantExpression(new Long(Long.MAX_VALUE)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple division results in null", result);
        Assert.assertEquals("Mixed (double,long) division does not return a double",
                            Double.class, result.getClass());
        Assert.assertEquals("Division doesn't divide properly",
                            42.1/Long.MAX_VALUE, ((Double)result).doubleValue(), 0.0);
    }

    public void testMixedDivisionDoubleFloat()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Double(42.1)),
                                                      "/",
                                                      new ConstantExpression(new Float(17.1)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple division results in null", result);
        Assert.assertEquals("Mixed (double,float) division does not return a double",
                            Double.class, result.getClass());
        Assert.assertEquals("Division doesn't divide properly",
                            42.1/17.1f, ((Double)result).doubleValue(), 0.0);
    }

    public void testMixedDivisionFloatInt()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Float(42.1)),
                                                      "/",
                                                      new ConstantExpression(new Integer(17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple division results in null", result);
        Assert.assertEquals("Mixed (float,int) division does not return a float",
                            Float.class, result.getClass());
        Assert.assertEquals("Division doesn't divide properly",
                            42.1/17, ((Float)result).floatValue(), 0.01);
    }


    public void testMixedDivisionFloatLong()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Float(42.1)),
                                                      "/",
                                                      new ConstantExpression(new Long(Long.MAX_VALUE)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple division results in null", result);
        Assert.assertEquals("Mixed (float,long) division does not return a float",
                            Float.class, result.getClass());
        Assert.assertEquals("Division doesn't divide properly",
                            (float)(42.1/Long.MAX_VALUE),
                            ((Float)result).floatValue(),
                            0.01f);
    }

    public void testMixedDivisionIntegerByte()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                      "/",
                                                      new ConstantExpression(new Byte((byte)17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple division results in null", result);
        Assert.assertEquals("Mixed (int,byte) division does not return an int",
                            Integer.class, result.getClass());
        Assert.assertEquals("Division doesn't divide properly",
                            42/17, ((Integer)result).intValue());
    }

    public void testNullsDivision()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(nullExp(),
                                                      "/",
                                                      nullExp());

        Assert.assertNull("Dividing nulls does not return null",
                          exp.evaluate(ec));
    }

    public void testLeftNullDivision()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(nullExp(),
                                                      "/",
                                                      new ConstantExpression(new Integer(1)));

        Assert.assertNull("Dividing left null does not return null",
                          exp.evaluate(ec));
    }

    public void testRightNullDivision()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(1)),
                                                      "/",
                                                      nullExp());

        Assert.assertNull("Dividing right null does not return null",
                          exp.evaluate(ec));
    }

    //
    // Modulus
    //

    public void testIntegerModulus()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                      "%",
                                                      new ConstantExpression(new Integer(17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple modulus results in null", result);
        Assert.assertEquals("Integer modulus does not return an integer",
                            Integer.class, result.getClass());
        Assert.assertEquals("Modulus doesn't mod properly",
                            42%17, ((Integer)result).intValue());
    }

    public void testLongModulus()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Long(42)),
                                                      "%",
                                                      new ConstantExpression(new Long(17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple modulus results in null", result);
        Assert.assertEquals("Long modulus does not return an integer",
                            Long.class, result.getClass());
        Assert.assertEquals("Modulus doesn't mod properly",
                            (42%17), ((Long)result).longValue());
    }

    public void testFloatModulus()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Float(42.1)),
                                                      "%",
                                                      new ConstantExpression(new Float(17.1)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple modulus results in null", result);
        Assert.assertEquals("Float modulus does not return a float",
                            Float.class, result.getClass());
        Assert.assertEquals("Modulus doesn't mod properly",
                            (float)(42.1%17.1), ((Float)result).floatValue(),
                            0.01f);
    }

    public void testDoubleModulus()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Double(42.1)),
                                                      "%",
                                                      new ConstantExpression(new Double(17.1)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple modulus results in null", result);
        Assert.assertEquals("Double modulus does not return a double",
                            Double.class, result.getClass());
        Assert.assertEquals("Modulus doesn't mod properly",
                            42.1%17.1, ((Double)result).doubleValue(), 0.0);
    }

    public void testMixedModulusDoubleInt()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Double(42.1)),
                                                      "%",
                                                      new ConstantExpression(new Integer(17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple modulus results in null", result);
        Assert.assertEquals("Mixed (double,int) modulus does not return a double",
                            Double.class, result.getClass());
        Assert.assertEquals("Modulus doesn't mod properly",
                            42.1%17, ((Double)result).doubleValue(), 0.0);
    }


    public void testMixedModulusDoubleLong()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Double(42.1)),
                                                      "%",
                                                      new ConstantExpression(new Long(Long.MAX_VALUE)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple modulus results in null", result);
        Assert.assertEquals("Mixed (double,long) modulus does not return a double",
                            Double.class, result.getClass());
        Assert.assertEquals("Modulus doesn't mod properly",
                            42.1%Long.MAX_VALUE, ((Double)result).doubleValue(), 0.0);
    }

    public void testMixedModulusDoubleFloat()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Double(42.1)),
                                                      "%",
                                                      new ConstantExpression(new Float(17.1)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple modulus results in null", result);
        Assert.assertEquals("Mixed (double,float) modulus does not return a double",
                            Double.class, result.getClass());
        Assert.assertEquals("Modulus doesn't mod properly",
                            42.1%17.1f, ((Double)result).doubleValue(), 0.0);
    }

    public void testMixedModulusFloatInt()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Float(42.1)),
                                                      "%",
                                                      new ConstantExpression(new Integer(17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple modulus results in null", result);
        Assert.assertEquals("Mixed (float,int) modulus does not return a float",
                            Float.class, result.getClass());
        Assert.assertEquals("Modulus doesn't mod properly",
                            42.1%17, ((Float)result).floatValue(), 0.01);
    }


    public void testMixedModulusFloatLong()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Float(42.1)),
                                                      "%",
                                                      new ConstantExpression(new Long(Long.MAX_VALUE)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple modulus results in null", result);
        Assert.assertEquals("Mixed (float,long) modulus does not return a float",
                            Float.class, result.getClass());
        Assert.assertEquals("Modulus doesn't mod properly",
                            (float)(42.1%Long.MAX_VALUE),
                            ((Float)result).floatValue(),
                            0.01f);
    }

    public void testMixedModulusIntegerByte()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                      "%",
                                                      new ConstantExpression(new Byte((byte)17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple modulus results in null", result);
        Assert.assertEquals("Mixed (int,byte) modulus does not return an int",
                            Integer.class, result.getClass());
        Assert.assertEquals("Modulus doesn't mod properly",
                            42%17, ((Integer)result).intValue());
    }

    public void testNullsModulus()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(nullExp(),
                                                      "%",
                                                      nullExp());

        Assert.assertNull("Modding nulls does not return null",
                          exp.evaluate(ec));
    }

    public void testLeftNullModulus()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(nullExp(),
                                                      "%",
                                                      new ConstantExpression(new Integer(1)));

        Assert.assertNull("Modding left null does not return null",
                          exp.evaluate(ec));
    }

    public void testRightNullModulus()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(1)),
                                                      "%",
                                                      nullExp());

        Assert.assertNull("Modding right null does not return null",
                          exp.evaluate(ec));
    }

    //
    //
    // ... and the rest
    //
    //

    public void testMultipleAddition()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                      "+",
                                                      new ConstantExpression(new Integer(17)));
        Expression exp2 = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                       "+",
                                                       new ConstantExpression(new Integer(17)));
        exp = new BinaryOperatorExpression(exp, "+", exp2);

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Multiple addition results in null", result);
        Assert.assertEquals("Integer addition does not return an integer",
                            Integer.class, result.getClass());
        Assert.assertEquals("Addition doesn't add properly",
                            42+17+42+17, ((Integer)result).intValue());
    }

    public void testMultipleSubtraction()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                      "-",
                                                      new ConstantExpression(new Integer(17)));
        Expression exp2 = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                       "-",
                                                       new ConstantExpression(new Integer(17)));
        exp = new BinaryOperatorExpression(exp, "-", exp2);

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Multiple subtraction results in null", result);
        Assert.assertEquals("Integer subtraction does not return an integer",
                            Integer.class, result.getClass());
        Assert.assertEquals("Subtraction doesn't add properly",
                            (42-17) - (42-17), ((Integer)result).intValue());
    }

    public void testDivision()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                      "/",
                                                      new ConstantExpression(new Integer(17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple division results in null", result);
        Assert.assertEquals("Integer division does not return an integer",
                            Integer.class, result.getClass());
        Assert.assertEquals("Division doesn't divide properly",
                            42/17, ((Integer)result).intValue());

        exp = new BinaryOperatorExpression(new ConstantExpression(new Float(42.0f)),
                                           "/",
                                           new ConstantExpression(new Float(17.0f)));

        result = exp.evaluate(ec);

        Assert.assertNotNull("Simple division results in null", result);
        Assert.assertEquals("Float division does not return an Float",
                            Float.class, result.getClass());
        Assert.assertEquals("Division doesn't divide properly",
                            42.0f/17.0f,
                            ((Float)result).floatValue(),
                            0f);

        exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(1)),
                                           "/",
                                           new ConstantExpression(new Float(2.0f)));

        result = exp.evaluate(ec);

        Assert.assertNotNull("Mixed division results in null", result);
        Assert.assertEquals("Mixed division does not return an Float",
                            Float.class, result.getClass());
        Assert.assertEquals("Division doesn't divide properly",
                            1/2.0f,
                            ((Float)result).floatValue(),
                            0f);

        exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(5)),
                                           "%",
                                           new ConstantExpression(new Integer(3)));

        result = exp.evaluate(ec);

        Assert.assertNotNull("Simple modulus results in null", result);
        Assert.assertEquals("Modulus does not return an Integer",
                            Integer.class, result.getClass());
        Assert.assertEquals("Modulus doesn't mod properly",
                            5 % 3,
                            ((Integer)result).floatValue(),
                            0f);

    }

    public void testMultipleMultiplication()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                      "*",
                                                      new ConstantExpression(new Integer(17)));
        Expression exp2 = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                       "*",
                                                       new ConstantExpression(new Integer(17)));
        exp = new BinaryOperatorExpression(exp, "*", exp2);

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Multiple multiplication results in null", result);
        Assert.assertEquals("Integer multiplication does not return an integer",
                            Integer.class, result.getClass());
        Assert.assertEquals("Multiplication doesn't multiply properly",
                            (42*17)*(42*17), ((Integer)result).intValue());
    }

    public void testMixedArithmetic()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                      "*",
                                                      new ConstantExpression(new Integer(17)));
        Expression exp2 = new BinaryOperatorExpression(new ConstantExpression(new Integer(14)),
                                                       "-",
                                                       new ConstantExpression(new Integer(24)));

        exp = new BinaryOperatorExpression(exp, "+", exp2);

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Mixed arithmetic results in null", result);
        Assert.assertEquals("Integer mixed arithmetic does not return an integer",
                            Integer.class, result.getClass());
        Assert.assertEquals("Mixed arithmetic doesn't evaluate properly",
                            (42*17) + (14-24), ((Integer)result).intValue());
    }

    //
    // Equality
    //

    public void testIntegerEquality()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                      "=",
                                                      new ConstantExpression(new Integer(17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple equality results in null", result);
        Assert.assertEquals("Equality does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Equality doesn't evaluate properly",
                            Boolean.FALSE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(12)),
                                           "=",
                                           new ConstantExpression(new Integer(12)));
        result = exp.evaluate(ec);

        Assert.assertNotNull("Simple equality results in null", result);
        Assert.assertEquals("Equality does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Equality doesn't evaluate properly",
                            Boolean.TRUE, result);
    }

    public void testStringEquality()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression("test"),
                                                      "=",
                                                      new ConstantExpression("test"));
        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple string equality results in null", result);
        Assert.assertEquals("Equality does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("String equality doesn't evaluate properly",
                            Boolean.TRUE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression("test"),
                                           "=",
                                           new ConstantExpression("test "));
        result = exp.evaluate(ec);

        Assert.assertNotNull("Simple string equality results in null", result);
        Assert.assertEquals("Equality does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("String equality doesn't evaluate properly",
                            Boolean.FALSE, result);
    }

    public void testMixedEqualityStringInt()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression("test"),
                                                      "=",
                                                      new ConstantExpression(new Integer(3)));
        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Mixed equality results in null", result);
        Assert.assertEquals("Mixed equality does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Mixed equality doesn't evaluate properly",
                            Boolean.FALSE, result);
    }

    public void testMixedEqualityLongInt()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Long(3)),
                                                      "=",
                                                      new ConstantExpression(new Integer(3)));
        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Mixed equality results in null", result);
        Assert.assertEquals("Mixed equality does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Mixed equality doesn't evaluate properly",
                            Boolean.TRUE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(3)),
                                           "=",
                                           new ConstantExpression(new Long(3)));
        result = exp.evaluate(ec);

        Assert.assertNotNull("Mixed equality results in null", result);
        Assert.assertEquals("Mixed equality does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Mixed equality doesn't evaluate properly",
                            Boolean.TRUE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(new Long(4)),
                                           "=",
                                           new ConstantExpression(new Integer(3)));
        result = exp.evaluate(ec);

        Assert.assertNotNull("Mixed equality results in null", result);
        Assert.assertEquals("Mixed equality does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Mixed equality doesn't evaluate properly",
                            Boolean.FALSE, result);
    }

    public void testMixedEqualityDoubleInt()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Double(3.0)),
                                                      "=",
                                                      new ConstantExpression(new Integer(3)));
        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Mixed equality results in null", result);
        Assert.assertEquals("Mixed equality does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Mixed equality doesn't evaluate properly",
                            Boolean.TRUE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(3)),
                                           "=",
                                           new ConstantExpression(new Double(3.0)));
        result = exp.evaluate(ec);

        Assert.assertNotNull("Mixed equality results in null", result);
        Assert.assertEquals("Mixed equality does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Mixed equality doesn't evaluate properly",
                            Boolean.TRUE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(new Double(4.0)),
                                           "=",
                                           new ConstantExpression(new Integer(3)));
        result = exp.evaluate(ec);

        Assert.assertNotNull("Mixed equality results in null", result);
        Assert.assertEquals("Mixed equality does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Mixed equality doesn't evaluate properly",
                            Boolean.FALSE, result);
    }

    public void testNullsEquals()
        throws EvaluationException
    {
        ec.setProperty(MathFunction.ALLOW_NULL_COMPARISON, Boolean.TRUE);
        Expression exp = new BinaryOperatorExpression(nullExp(),
                                                      "=",
                                                      nullExp());

        Assert.assertEquals("Equaling nulls does not return TRUE",
                            Boolean.TRUE, exp.evaluate(ec));

        ec.setProperty(MathFunction.ALLOW_NULL_COMPARISON, Boolean.FALSE);
        Assert.assertNull("Equaling nulls does not return null",
                          exp.evaluate(ec));
    }
			    

    public void testLeftNullEquals()
        throws EvaluationException
    {
        ec.setProperty(MathFunction.ALLOW_NULL_COMPARISON, Boolean.TRUE);
        Expression exp = new BinaryOperatorExpression(nullExp(),
                                                      "=",
                                                      new ConstantExpression(new Integer(1)));

        Assert.assertEquals("Equaling left null does not return FALSE",
                            Boolean.FALSE, exp.evaluate(ec));

        ec.setProperty(MathFunction.ALLOW_NULL_COMPARISON, Boolean.FALSE);
        Assert.assertNull("Equaling nulls does not return null",
                          exp.evaluate(ec));
    }

    public void testRightNullEquals()
        throws EvaluationException
    {
        ec.setProperty(MathFunction.ALLOW_NULL_COMPARISON, Boolean.TRUE);
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(1)),
                                                      "=",
                                                      nullExp());

        Assert.assertEquals("Equaling right null does not return FALSE",
                            Boolean.FALSE, exp.evaluate(ec));

        ec.setProperty(MathFunction.ALLOW_NULL_COMPARISON, Boolean.FALSE);
        Assert.assertNull("Equaling nulls does not return null",
                          exp.evaluate(ec));
    }

    //
    // Less than
    //

    public void testIntegerLessthan()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                      "<",
                                                      new ConstantExpression(new Integer(17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple lessthan results in null", result);
        Assert.assertEquals("Lessthan does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Lessthan doesn't evaluate properly",
                            Boolean.FALSE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(17)),
                                           "<",
                                           new ConstantExpression(new Integer(42)));
        result = exp.evaluate(ec);

        Assert.assertNotNull("Simple lessthan results in null", result);
        Assert.assertEquals("Lessthan does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Lessthan doesn't evaluate properly",
                            Boolean.TRUE, result);
    }

    public void testStringLessthan()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression("test"),
                                                      "<",
                                                      new ConstantExpression("test"));
        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple string lessthan results in null", result);
        Assert.assertEquals("Lessthan does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("String lessthan doesn't evaluate properly",
                            Boolean.FALSE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression("test"),
                                           "<",
                                           new ConstantExpression("test "));
        result = exp.evaluate(ec);

        Assert.assertNotNull("Simple string lessthan results in null", result);
        Assert.assertEquals("Lessthan does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("String lessthan doesn't evaluate properly",
                            Boolean.TRUE, result);
    }

    public void testMixedLessthanStringInt()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression("test"),
                                                      "<",
                                                      new ConstantExpression(new Integer(3)));
        try
        {
            exp.evaluate(ec);

            Assert.fail("Incompatible less-than doesn't fail");
        }
        catch (BinaryOperator.IncompatibleArgumentsException iae)
        {
            // Expected behavior
        }
    }

    public void testMixedLessthanLongInt()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Long(3)),
                                                      "<",
                                                      new ConstantExpression(new Integer(5)));
        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Mixed lessthan results in null", result);
        Assert.assertEquals("Mixed lessthan does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Mixed lessthan doesn't evaluate properly",
                            Boolean.TRUE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(3)),
                                           "<",
                                           new ConstantExpression(new Long(4)));
        result = exp.evaluate(ec);

        Assert.assertNotNull("Mixed lessthan results in null", result);
        Assert.assertEquals("Mixed lessthan does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Mixed lessthan doesn't evaluate properly",
                            Boolean.TRUE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(new Long(4)),
                                           "<",
                                           new ConstantExpression(new Integer(3)));
        result = exp.evaluate(ec);

        Assert.assertNotNull("Mixed lessthan results in null", result);
        Assert.assertEquals("Mixed lessthan does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Mixed lessthan doesn't evaluate properly",
                            Boolean.FALSE, result);
    }

    public void testMixedLessthanDoubleInt()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Double(3.0)),
                                                      "<",
                                                      new ConstantExpression(new Integer(4)));
        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Mixed lessthan results in null", result);
        Assert.assertEquals("Mixed lessthan does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Mixed lessthan doesn't evaluate properly",
                            Boolean.TRUE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(3)),
                                           "<",
                                           new ConstantExpression(new Double(4.0)));
        result = exp.evaluate(ec);

        Assert.assertNotNull("Mixed lessthan results in null", result);
        Assert.assertEquals("Mixed lessthan does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Mixed lessthan doesn't evaluate properly",
                            Boolean.TRUE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(new Double(4.0)),
                                           "<",
                                           new ConstantExpression(new Integer(3)));
        result = exp.evaluate(ec);

        Assert.assertNotNull("Mixed lessthan results in null", result);
        Assert.assertEquals("Mixed lessthan does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Mixed lessthan doesn't evaluate properly",
                            Boolean.FALSE, result);
    }

    public void testNullsLessthan()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(nullExp(),
                                                      "<",
                                                      nullExp());

        Assert.assertNull("Lessing nulls does not return null",
                          exp.evaluate(ec));
    }

    public void testLeftNullLessthan()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(nullExp(),
                                                      "<",
                                                      new ConstantExpression(new Integer(1)));

        Assert.assertNull("Lessing left null does not return null",
                          exp.evaluate(ec));
    }

    public void testRightNullLessthan()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(1)),
                                                      "<",
                                                      nullExp());

        Assert.assertNull("Lessing right null does not return null",
                          exp.evaluate(ec));
    }

    //
    // Greater than
    //

    public void testIntegerGreaterthan()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                      ">",
                                                      new ConstantExpression(new Integer(17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple greaterthan results in null", result);
        Assert.assertEquals("Greaterthan does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Greaterthan doesn't evaluate properly",
                            Boolean.TRUE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(17)),
                                           ">",
                                           new ConstantExpression(new Integer(42)));
        result = exp.evaluate(ec);

        Assert.assertNotNull("Simple greaterthan results in null", result);
        Assert.assertEquals("Greaterthan does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Greaterthan doesn't evaluate properly",
                            Boolean.FALSE, result);
    }

    public void testStringGreaterthan()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression("test"),
                                                      ">",
                                                      new ConstantExpression("test"));
        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple string greaterthan results in null", result);
        Assert.assertEquals("Greaterthan does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("String greaterthan doesn't evaluate properly",
                            Boolean.FALSE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression("test"),
                                           ">",
                                           new ConstantExpression("test "));
        result = exp.evaluate(ec);

        Assert.assertNotNull("Simple string greaterthan results in null", result);
        Assert.assertEquals("Greaterthan does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("String greaterthan doesn't evaluate properly",
                            Boolean.FALSE, result);
    }

    public void testMixedGreaterthanStringInt()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression("test"),
                                                      ">",
                                                      new ConstantExpression(new Integer(3)));
        try
        {
            exp.evaluate(ec);

            Assert.fail("Incompatible less-than doesn't fail");
        }
        catch (BinaryOperator.IncompatibleArgumentsException iae)
        {
            // Expected behavior
        }
    }

    public void testMixedGreaterthanLongInt()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Long(3)),
                                                      ">",
                                                      new ConstantExpression(new Integer(5)));
        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Mixed greaterthan results in null", result);
        Assert.assertEquals("Mixed greaterthan does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Mixed greaterthan doesn't evaluate properly",
                            Boolean.FALSE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(3)),
                                           ">",
                                           new ConstantExpression(new Long(4)));
        result = exp.evaluate(ec);

        Assert.assertNotNull("Mixed greaterthan results in null", result);
        Assert.assertEquals("Mixed greaterthan does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Mixed greaterthan doesn't evaluate properly",
                            Boolean.FALSE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(new Long(4)),
                                           ">",
                                           new ConstantExpression(new Integer(3)));
        result = exp.evaluate(ec);

        Assert.assertNotNull("Mixed greaterthan results in null", result);
        Assert.assertEquals("Mixed greaterthan does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Mixed greaterthan doesn't evaluate properly",
                            Boolean.TRUE, result);
    }

    public void testMixedGreaterthanDoubleInt()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Double(3.0)),
                                                      ">",
                                                      new ConstantExpression(new Integer(4)));
        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Mixed greaterthan results in null", result);
        Assert.assertEquals("Mixed greaterthan does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Mixed greaterthan doesn't evaluate properly",
                            Boolean.FALSE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(3)),
                                           ">",
                                           new ConstantExpression(new Double(4.0)));
        result = exp.evaluate(ec);

        Assert.assertNotNull("Mixed greaterthan results in null", result);
        Assert.assertEquals("Mixed greaterthan does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Mixed greaterthan doesn't evaluate properly",
                            Boolean.FALSE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(new Double(4.0)),
                                           ">",
                                           new ConstantExpression(new Integer(3)));
        result = exp.evaluate(ec);

        Assert.assertNotNull("Mixed greaterthan results in null", result);
        Assert.assertEquals("Mixed greaterthan does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Mixed greaterthan doesn't evaluate properly",
                            Boolean.TRUE, result);
    }

    public void testNullsGreaterthan()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(nullExp(),
                                                      ">",
                                                      nullExp());

        Assert.assertNull("Greatering nulls does not return null",
                          exp.evaluate(ec));
    }

    public void testLeftNullGreaterthan()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(nullExp(),
                                                      ">",
                                                      new ConstantExpression(new Integer(1)));

        Assert.assertNull("Greatering left null does not return null",
                          exp.evaluate(ec));
    }

    public void testRightNullGreaterthan()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(1)),
                                                      ">",
                                                      nullExp());

        Assert.assertNull("Greatering right null does not return null",
                          exp.evaluate(ec));
    }

    //
    // ...and the rest...
    //

    public void testRelationals()
        throws EvaluationException
    {
        // Test greater-than
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                      ">",
                                                      new ConstantExpression(new Integer(17)));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple relation results in null", result);
        Assert.assertEquals("Relation does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Relation doesn't evaluate properly",
                            Boolean.TRUE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(17)),
                                           ">",
                                           new ConstantExpression(new Integer(42)));

        result = exp.evaluate(ec);

        Assert.assertNotNull("Simple relation results in null", result);
        Assert.assertEquals("Relation does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Relation doesn't evaluate properly",
                            Boolean.FALSE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(17)),
                                           ">",
                                           new ConstantExpression(new Integer(17)));

        result = exp.evaluate(ec);

        Assert.assertNotNull("Simple relation results in null", result);
        Assert.assertEquals("Relation does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Relation doesn't evaluate properly",
                            Boolean.FALSE, result);

        // Test less-than operator

        exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(17)),
                                           "<",
                                           new ConstantExpression(new Integer(17)));

        result = exp.evaluate(ec);

        Assert.assertNotNull("Simple relation results in null", result);
        Assert.assertEquals("Relation does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Relation doesn't evaluate properly",
                            Boolean.FALSE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(17)),
                                           "<",
                                           new ConstantExpression(new Integer(42)));

        result = exp.evaluate(ec);

        Assert.assertNotNull("Simple relation results in null", result);
        Assert.assertEquals("Relation does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Relation doesn't evaluate properly",
                            Boolean.TRUE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                           "<",
                                           new ConstantExpression(new Integer(17)));

        result = exp.evaluate(ec);

        Assert.assertNotNull("Simple relation results in null", result);
        Assert.assertEquals("Relation does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Relation doesn't evaluate properly",
                            Boolean.FALSE, result);

        // Test greater-than-or-equal-to
        exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                           ">=",
                                           new ConstantExpression(new Integer(17)));

        result = exp.evaluate(ec);

        Assert.assertNotNull("Simple relation results in null", result);
        Assert.assertEquals("Relation does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Relation doesn't evaluate properly",
                            Boolean.TRUE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(17)),
                                           ">=",
                                           new ConstantExpression(new Integer(42)));

        result = exp.evaluate(ec);

        Assert.assertNotNull("Simple relation results in null", result);
        Assert.assertEquals("Relation does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Relation doesn't evaluate properly",
                            Boolean.FALSE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(17)),
                                           ">=",
                                           new ConstantExpression(new Integer(17)));

        result = exp.evaluate(ec);

        Assert.assertNotNull("Simple relation results in null", result);
        Assert.assertEquals("Relation does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Relation doesn't evaluate properly",
                            Boolean.TRUE, result);

        // Test less-than-or-equal-to operator
        exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(17)),
                                           "<=",
                                           new ConstantExpression(new Integer(17)));

        result = exp.evaluate(ec);

        Assert.assertNotNull("Simple relation results in null", result);
        Assert.assertEquals("Relation does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Relation doesn't evaluate properly",
                            Boolean.TRUE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(17)),
                                           "<=",
                                           new ConstantExpression(new Integer(42)));

        result = exp.evaluate(ec);

        Assert.assertNotNull("Simple relation results in null", result);
        Assert.assertEquals("Relation does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Relation doesn't evaluate properly",
                            Boolean.TRUE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                           "<=",
                                           new ConstantExpression(new Integer(17)));

        result = exp.evaluate(ec);

        Assert.assertNotNull("Simple relation results in null", result);
        Assert.assertEquals("Relation does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Relation doesn't evaluate properly",
                            Boolean.FALSE, result);
    }

    public void testEqualsNull()
        throws EvaluationException
    {
        ec.setProperty(MathFunction.ALLOW_NULL_COMPARISON, Boolean.FALSE);

        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                      "=",
                                                      new IdentifierExpression("null"));

        Object result = exp.evaluate(ec);

        Assert.assertNull("Null comparison does not yield null", result);

        ec.setProperty(MathFunction.ALLOW_NULL_COMPARISON, Boolean.TRUE);

        result = exp.evaluate(ec);
        Assert.assertEquals("Null comparison does not yield boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Null comparison does not yield FALSE",
                            Boolean.FALSE, result);

        exp = new BinaryOperatorExpression(new IdentifierExpression("null"),
                                           "=",
                                           new IdentifierExpression("null"));

        result = exp.evaluate(ec);
        Assert.assertEquals("Null comparison does not yield boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Null comparison does not yield TRUE",
                            Boolean.TRUE, result);
    }

    public void testNotEqualsNull()
        throws EvaluationException
    {
        ec.setProperty(MathFunction.ALLOW_NULL_COMPARISON, Boolean.FALSE);

        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                      "!=",
                                                      new IdentifierExpression("null"));

        Object result = exp.evaluate(ec);

        Assert.assertNull("Null comparison does not yield null", result);

        ec.setProperty(MathFunction.ALLOW_NULL_COMPARISON, Boolean.TRUE);

        result = exp.evaluate(ec);
        Assert.assertNotNull("Null comparison returns null", result);
        Assert.assertEquals("Null comparison does not yield boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Null comparison does not yield TRUE",
                            Boolean.TRUE, result);

        exp = new BinaryOperatorExpression(new IdentifierExpression("null"),
                                           "!=",
                                           new IdentifierExpression("null"));

        result = exp.evaluate(ec);
        Assert.assertNotNull("Null comparison returns null", result);
        Assert.assertEquals("Null comparison does not yield boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Null comparison does not yield FALSE",
                            Boolean.FALSE, result);
    }

    public void testGreaterNull()
        throws EvaluationException
    {
        ec.setProperty(MathFunction.ALLOW_NULL_COMPARISON, Boolean.FALSE);

        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                      ">",
                                                      new IdentifierExpression("null"));

        Object result = exp.evaluate(ec);

        Assert.assertNull("Null comparison does not yield null", result);

        ec.setProperty(MathFunction.ALLOW_NULL_COMPARISON, Boolean.TRUE);

        result = exp.evaluate(ec);
        Assert.assertNotNull("Null comparison returns null", result);
        Assert.assertEquals("Null comparison does not yield boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Null comparison does not yield FALSE",
                            Boolean.FALSE, result);

        exp = new BinaryOperatorExpression(new IdentifierExpression("null"),
                                           ">",
                                           new IdentifierExpression("null"));

        result = exp.evaluate(ec);
        Assert.assertNotNull("Null comparison returns null", result);
        Assert.assertEquals("Null comparison does not yield boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Null comparison does not yield FALSE",
                            Boolean.FALSE, result);
    }

    public void testLessNull()
        throws EvaluationException
    {
        ec.setProperty(MathFunction.ALLOW_NULL_COMPARISON, Boolean.FALSE);

        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                      "<",
                                                      new IdentifierExpression("null"));

        Object result = exp.evaluate(ec);

        Assert.assertNull("Null comparison does not yield null", result);

        ec.setProperty(MathFunction.ALLOW_NULL_COMPARISON, Boolean.TRUE);

        result = exp.evaluate(ec);
        Assert.assertNotNull("Null comparison returns null", result);
        Assert.assertEquals("Null comparison does not yield boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Null comparison does not yield FALSE",
                            Boolean.FALSE, result);

        exp = new BinaryOperatorExpression(new IdentifierExpression("null"),
                                           "<",
                                           new IdentifierExpression("null"));

        result = exp.evaluate(ec);
        Assert.assertNotNull("Null comparison returns null", result);
        Assert.assertEquals("Null comparison does not yield boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Null comparison does not yield FALSE",
                            Boolean.FALSE, result);
    }

    public void testGreaterEqualNull()
        throws EvaluationException
    {
        ec.setProperty(MathFunction.ALLOW_NULL_COMPARISON, Boolean.FALSE);

        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                      ">=",
                                                      new IdentifierExpression("null"));

        Object result = exp.evaluate(ec);

        Assert.assertNull("Null comparison does not yield null", result);

        ec.setProperty(MathFunction.ALLOW_NULL_COMPARISON, Boolean.TRUE);

        result = exp.evaluate(ec);
        Assert.assertNotNull("Null comparison returns null", result);
        Assert.assertEquals("Null comparison does not yield boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Null comparison does not yield FALSE",
                            Boolean.FALSE, result);

        exp = new BinaryOperatorExpression(new IdentifierExpression("null"),
                                           ">=",
                                           new IdentifierExpression("null"));

        result = exp.evaluate(ec);
        Assert.assertNotNull("Null comparison returns null", result);
        Assert.assertEquals("Null comparison does not yield boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Null comparison does not yield TRUE",
                            Boolean.TRUE, result);
    }

    public void testLessEqualNull()
        throws EvaluationException
    {
        ec.setProperty(MathFunction.ALLOW_NULL_COMPARISON, Boolean.FALSE);

        Expression exp = new BinaryOperatorExpression(new ConstantExpression(new Integer(42)),
                                                      "<=",
                                                      new IdentifierExpression("null"));

        Object result = exp.evaluate(ec);

        Assert.assertNull("Null comparison does not yield null", result);

        ec.setProperty(MathFunction.ALLOW_NULL_COMPARISON, Boolean.TRUE);

        result = exp.evaluate(ec);
        Assert.assertNotNull("Null comparison returns null", result);
        Assert.assertEquals("Null comparison does not yield boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Null comparison does not yield FALSE",
                            Boolean.FALSE, result);

        exp = new BinaryOperatorExpression(new IdentifierExpression("null"),
                                           "<=",
                                           new IdentifierExpression("null"));

        result = exp.evaluate(ec);
        Assert.assertNotNull("Null comparison returns null", result);
        Assert.assertEquals("Null comparison does not yield boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("Null comparison does not yield TRUE",
                            Boolean.TRUE, result);
    }

    public void testLogicals()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(Boolean.TRUE),
                                                      "&&",
                                                      new ConstantExpression(Boolean.TRUE));

        Object result = exp.evaluate(ec);

        Assert.assertNotNull("Simple AND results in null", result);
        Assert.assertEquals("Simple logical does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("AND doesn't evaluate properly",
                            Boolean.TRUE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(Boolean.FALSE),
                                           "&&",
                                           new ConstantExpression(Boolean.TRUE));

        result = exp.evaluate(ec);

        Assert.assertNotNull("Simple AND results in null", result);
        Assert.assertEquals("Simple logical does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("AND doesn't evaluate properly",
                            Boolean.FALSE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(Boolean.TRUE),
                                           "&&",
                                           new ConstantExpression(Boolean.FALSE));

        result = exp.evaluate(ec);

        Assert.assertNotNull("Simple AND results in null", result);
        Assert.assertEquals("Simple logical does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("AND doesn't evaluate properly",
                            Boolean.FALSE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(Boolean.FALSE),
                                           "&&",
                                           new ConstantExpression(Boolean.FALSE));

        result = exp.evaluate(ec);

        Assert.assertNotNull("Simple AND results in null", result);
        Assert.assertEquals("Simple logical does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("AND doesn't evaluate properly",
                            Boolean.FALSE, result);

        //
        // OR
        //

        exp = new BinaryOperatorExpression(new ConstantExpression(Boolean.TRUE),
                                           "||",
                                           new ConstantExpression(Boolean.TRUE));

        result = exp.evaluate(ec);

        Assert.assertNotNull("Simple OR results in null", result);
        Assert.assertEquals("Simple logical does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("OR doesn't evaluate properly",
                            Boolean.TRUE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(Boolean.FALSE),
                                           "||",
                                           new ConstantExpression(Boolean.TRUE));

        result = exp.evaluate(ec);

        Assert.assertNotNull("Simple OR results in null", result);
        Assert.assertEquals("Simple logical does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("OR doesn't evaluate properly",
                            Boolean.TRUE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(Boolean.TRUE),
                                           "||",
                                           new ConstantExpression(Boolean.FALSE));

        result = exp.evaluate(ec);

        Assert.assertNotNull("Simple OR results in null", result);
        Assert.assertEquals("Simple logical does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("OR doesn't evaluate properly",
                            Boolean.TRUE, result);

        exp = new BinaryOperatorExpression(new ConstantExpression(Boolean.FALSE),
                                           "||",
                                           new ConstantExpression(Boolean.FALSE));

        result = exp.evaluate(ec);

        Assert.assertNotNull("Simple OR results in null", result);
        Assert.assertEquals("Simple logical does not return a boolean",
                            Boolean.class, result.getClass());
        Assert.assertEquals("OR doesn't evaluate properly",
                            Boolean.FALSE, result);
    }

    public void testNullsAnd()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(nullExp(),
                                                      "&&",
                                                      nullExp());

        Assert.assertNull("Anding nulls does not return null",
                          exp.evaluate(ec));
    }

    public void testLeftNullAnd()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(nullExp(),
                                                      "&&",
                                                      new ConstantExpression(Boolean.TRUE));

        Assert.assertNull("Anding left null does not return null",
                          exp.evaluate(ec));
    }

    public void testRightNullAnd()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(Boolean.TRUE),
                                                      "&&",
                                                      nullExp());

        Assert.assertNull("Anding right null does not return null",
                          exp.evaluate(ec));
    }

    public void testNullsOr()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(nullExp(),
                                                      "||",
                                                      nullExp());

        Assert.assertNull("Oring nulls does not return null",
                          exp.evaluate(ec));
    }

    public void testLeftNullOr()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(nullExp(),
                                                      "||",
                                                      new ConstantExpression(Boolean.TRUE));

        Assert.assertNull("Oring left null does not return null",
                          exp.evaluate(ec));
    }

    public void testRightNullOr()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression(Boolean.TRUE),
                                                      "||",
                                                      nullExp());

        Assert.assertNull("Oring right null does not return null",
                          exp.evaluate(ec));
    }

    public void testStringConcatenation()
        throws EvaluationException
    {
        Expression exp = new BinaryOperatorExpression(new ConstantExpression("a"),
                                                      "+",
                                                      new ConstantExpression("b"));

        Assert.assertEquals("String concatentation fails",
                            "ab",
                            exp.evaluate(ec));
    }

    /**
     * Creates a new UnitTest to execute the specified test.<p>
     *
     * @param name The name of the test method to run.
     */
    public BinaryOperatorTests(String name)
    {
        super(name);
    }
}
