/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.function;

import java.util.ArrayList;

import junit.framework.Assert;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import net.christopherschultz.evaluator.EvaluationContext;
import net.christopherschultz.evaluator.EvaluationException;
import net.christopherschultz.evaluator.Expression;
import net.christopherschultz.evaluator.parser.ExpressionParser;
import net.christopherschultz.evaluator.parser.ParseException;
import net.christopherschultz.evaluator.util.DefaultEvaluationContext;

/**
 * Unit test for the countMatches function.
 *
 * @author Chris Schultz
 * @version $Revision: 1.2 $ $Date: 2008-06-20 22:58:48 $
 */
public class CountMatchesTests
    extends TestCase
{
    EvaluationContext ec;

    private static final Integer ZERO = new Integer(0);
    private static final Integer ONE = new Integer(1);

    public void setUp()
    {
	ec = new DefaultEvaluationContext();

	ec.set("countMatches", new CountMatchesFunction());
	ec.set("set", new SetVariableFunction());
	ec.set("true", Boolean.TRUE);
	ec.set("false", Boolean.FALSE);
    }

    private Object evaluate(String expression,
			    EvaluationContext ec)
	throws EvaluationException, ParseException
    {
	Expression exp = ExpressionParser.parseExpression(expression);

	return exp.evaluate(ec);
    }

    public void testEmptyArray()
	throws Exception
    {
	ec.set("a", new Integer[0]);

	Assert.assertEquals(new Integer(0),
			    evaluate("countMatches(a, 1, 2, 3)", ec));
    }

    public void testEmptyCollection()
	throws Exception
    {
	ec.set("a", new ArrayList());

	Assert.assertEquals(new Integer(0),
			    evaluate("countMatches(a, 1, 2, 3)", ec));
    }

    public void testNullArray()
	throws Exception
    {
	Assert.assertEquals(new Integer(0),
			    evaluate("countMatches(a, 1, 2, 3)", ec));
    }

    public void testNullCollection()
	throws Exception
    {
	Assert.assertEquals(new Integer(0),
			    evaluate("countMatches(a, 1, 2, 3)", ec));
    }

    public void testArray()
	throws Exception
    {
	ec.set("a", new Integer[] { new Integer(1), new Integer(2), new Integer(3) });

	Assert.assertEquals(new Integer(3),
			    evaluate("countMatches(a, 1, 2, 3)", ec));
    }

    public void testPrimitiveArray()
	throws Exception
    {
	ec.set("a", new int[] { 1, 2, 3 });

	Assert.assertEquals(new Integer(3),
			    evaluate("countMatches(a, 1, 2, 3)", ec));
    }

    public void testCollection()
	throws Exception
    {
	ArrayList l = new ArrayList();
	l.add(new Integer(1));
	l.add(new Integer(2));
	l.add(new Integer(3));

	ec.set("a", l);

	Assert.assertEquals(new Integer(3),
			    evaluate("countMatches(a, 1, 2, 3)", ec));
    }

    public void testSome()
	throws Exception
    {
	ec.set("a", new Integer[] { new Integer(1), new Integer(3) });

	Assert.assertEquals(new Integer(2),
			    evaluate("countMatches(a, 1, 2, 3)", ec));
    }

    public void testNone()
	throws Exception
    {
	ec.set("a", new Integer[] { new Integer(4), new Integer(5) });

	Assert.assertEquals(new Integer(0),
			    evaluate("countMatches(a, 1, 2, 3)", ec));
    }

    public void testExtras()
	throws Exception
    {
	ec.set("a", new Integer[] { new Integer(0), new Integer(1), new Integer(2), new Integer(3), new Integer(4) });

	Assert.assertEquals(new Integer(3),
			    evaluate("countMatches(a, 1, 2, 3)", ec));
    }

    /**
     * Creates a new UnitTest to execute the specified test.<p>
     *
     * @param name The name of the test method to run.
     */
    public CountMatchesTests(String name)
    {
        super(name);
    }
}
