/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.function;

import junit.framework.Assert;
import junit.framework.Test;
import junit.framework.TestSuite;

import org.jmock.MockObjectTestCase;
import org.jmock.Mock;

import java.util.ArrayList;

import net.christopherschultz.evaluator.*;
import net.christopherschultz.evaluator.util.DefaultEvaluationContext;

/**
 * UnitTest for IfFunction.
 *
 * @author Chris Schultz
 * @version $Revision: 1.2 $ $Date: 2008-06-20 22:58:48 $
 */
public class IfFunctionTests
    extends net.christopherschultz.evaluator.UnitTest
{
    EvaluationContext ec;
    Expression exp;
    Mock f_true;
    Mock f_false;
    Object result;

    public void setUp()
    {
	f_true = new Mock(Function.class);
	f_false = new Mock(Function.class);

	ec = new DefaultEvaluationContext();

	ec.set("true", (Function)f_true.proxy());
	ec.set("false", (Function)f_false.proxy());
	ec.set("if", new IfFunction());
    }

    public void testEvaluation()
	throws EvaluationException
    {
	ArrayList args = new ArrayList();
	args.add(new FunctionCallExpression(id("true"), null));
	args.add(new FunctionCallExpression(id("true"), null));
	args.add(new FunctionCallExpression(id("false"), null));
	exp = new FunctionCallExpression(id("if"), args);

	f_true.expects(once()).method("call").with(eq(ec), ANYTHING, eq(null)).will(returnValue(Boolean.TRUE));
	f_true.expects(once()).method("call").with(eq(ec), ANYTHING, eq(null)).will(returnValue(Boolean.TRUE));
	f_false.expects(never()).method("call");

	result = exp.evaluate(ec);

	Assert.assertEquals("IfFunction returns incorrect branch.",
			   Boolean.TRUE, result);
    }

    public void testEvaluationAgain()
	throws EvaluationException
    {
	ArrayList args = new ArrayList();
	args.add(new FunctionCallExpression(id("false"), null));
	args.add(new FunctionCallExpression(id("true"), null));
	args.add(new FunctionCallExpression(id("false"), null));
	exp = new FunctionCallExpression(id("if"), args);

	f_false.expects(once()).method("call").with(eq(ec), ANYTHING, eq(null)).will(returnValue(Boolean.FALSE));
	f_false.expects(once()).method("call").with(eq(ec), ANYTHING, eq(null)).will(returnValue(Boolean.FALSE));
	f_true.expects(never()).method("call");

	result = exp.evaluate(ec);

	Assert.assertEquals("IfFunction returns incorrect branch.",
			   Boolean.FALSE, result);
    }

    public void testInvalidPredicate()
	throws EvaluationException
    {
	ArrayList args = new ArrayList();
	args.add(constant("foo"));
	args.add(id("bar"));
	args.add(id("baz"));
	exp = new FunctionCallExpression(id("if"), args);

	try
	{
	    result = exp.evaluate(ec);

	    Assert.fail("IfFunction should throw exception with non-boolean predicate.");
	}
	catch (UnexpectedArgumentException eae)
	{
	    // Expected behavior.
	}
    }

    public void testNullPredicate()
	throws EvaluationException
    {
	ArrayList args = new ArrayList();
	args.add(id("null"));
	args.add(constant("bar"));
	args.add(constant("baz"));
	exp = new FunctionCallExpression(id("if"), args);

	result = exp.evaluate(ec);

	Assert.assertNull("Null predicate should result in null result",
			  result);
    }


    public void testIncorrectArguments()
	throws EvaluationException
    {
	// Too few arguments
	exp = new FunctionCallExpression(id("if"), null);

	try
	{
	    result = exp.evaluate(ec);

	    Assert.fail("IfFunction should throw an exception when argument count is wrong.");
	}
	catch (EvaluationException ee)
	{
	    // Expected behavior
	}

	// Too few arguments
	ArrayList args = new ArrayList();
	args.add(id("foo"));
	args.add(id("bar"));
	exp = new FunctionCallExpression(id("if"), args);

	try
	{
	    result = exp.evaluate(ec);

	    Assert.fail("IfFunction should throw an exception when argument count is wrong.");
	}
	catch (EvaluationException ee)
	{
	    // Expected behavior
	}

	// Too many arguments
	args.clear();

	args.add(id("foo")); // remember, we already have 2 args in there
	args.add(id("bar")); // remember, we already have 2 args in there
	args.add(id("baz")); // remember, we already have 2 args in there
	args.add(id("ec"));
	exp = new FunctionCallExpression(id("if"), args);

	try
	{
	    result = exp.evaluate(ec);

	    Assert.fail("IfFunction should throw an exception when argument count is wrong.");
	}
	catch (EvaluationException ee)
	{
	    // Expected behavior
	}
    }

    /**
     * Returns a Test which will execute all the test methods found
     * in this class.<p>
     *
     * @return A Test for JUnit to run.
     */
    public static Test suite()
    {
        TestSuite ts = new TestSuite(IfFunctionTests.class);

        return ts;
    }

    /**
     * Creates a new UnitTest to execute the specified test.<p>
     *
     * @param name The name of the test method to run.
     */
    public IfFunctionTests(String name)
    {
        super(name);
    }
}
