/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.function;

import java.util.Arrays;
import java.util.ArrayList;

import junit.framework.Assert;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import net.christopherschultz.evaluator.ConstantExpression;
import net.christopherschultz.evaluator.EvaluationContext;
import net.christopherschultz.evaluator.EvaluationException;
import net.christopherschultz.evaluator.Expression;
import net.christopherschultz.evaluator.FunctionCallExpression;
import net.christopherschultz.evaluator.IdentifierExpression;
import net.christopherschultz.evaluator.parser.ExpressionParser;
import net.christopherschultz.evaluator.parser.ParseException;
import net.christopherschultz.evaluator.util.DefaultEvaluationContext;

/**
 * Unit test for the length function.
 *
 * @author Chris Schultz
 * @version $Revision: 1.2 $ $Date: 2008-06-20 22:58:48 $
 */
public class LengthTests
    extends TestCase
{
    EvaluationContext ec;

    public void setUp()
    {
        ec = new DefaultEvaluationContext();

        ec.set("length", new LengthFunction());
    }

    public void testEmptyArray()
        throws Exception
    {
        ec.set("a", new Integer[0]);

        Assert.assertEquals(new Integer(0),
        new FunctionCallExpression(new IdentifierExpression("length"),
                                   Arrays.asList(new Object[] {
                                       new IdentifierExpression("a")
                                   }))
                            .evaluate(ec));
    }

    public void testEmptyCollection()
        throws Exception
    {
        ec.set("a", new ArrayList());

        Assert.assertEquals(new Integer(0),
        new FunctionCallExpression(new IdentifierExpression("length"),
                                   Arrays.asList(new Object[] {
                                       new IdentifierExpression("a")
                                   }))
                            .evaluate(ec));
    }

    public void testNullArray()
        throws Exception
    {
        // leave "a" => null
        ec.setProperty(MathFunction.NULL_RETURNS_NULL, Boolean.FALSE);

        Assert.assertEquals(new Integer(0),
        new FunctionCallExpression(new IdentifierExpression("length"),
                                   Arrays.asList(new Object[] {
                                       new IdentifierExpression("a")
                                   }))
                            .evaluate(ec));
    }

    public void testNullArrayNull()
        throws Exception
    {
        // leave "a" => null
        ec.setProperty(MathFunction.NULL_RETURNS_NULL, Boolean.TRUE);

        Assert.assertNull(
        new FunctionCallExpression(new IdentifierExpression("length"),
                                   Arrays.asList(new Object[] {
                                       new IdentifierExpression("a")
                                   }))
                            .evaluate(ec));
    }

    public void testArray()
        throws Exception
    {
        ec.set("a", new Integer[] { new Integer(1), new Integer(2), new Integer(3) });

        Assert.assertEquals(new Integer(3),
        new FunctionCallExpression(new IdentifierExpression("length"),
                                   Arrays.asList(new Object[] {
                                       new IdentifierExpression("a")
                                   }))
                            .evaluate(ec));
    }

    public void testPrimitiveArray()
        throws Exception
    {
        ec.set("a", new int[] { 1, 2, 3 });

        Assert.assertEquals(new Integer(3),
        new FunctionCallExpression(new IdentifierExpression("length"),
                                   Arrays.asList(new Object[] {
                                       new IdentifierExpression("a")
                                   }))
                            .evaluate(ec));
    }

    public void testCollection()
        throws Exception
    {
        ArrayList l = new ArrayList();
        l.add(new Integer(1));
        l.add(new Integer(2));
        l.add(new Integer(3));

        ec.set("a", l);

        Assert.assertEquals(new Integer(3),
        new FunctionCallExpression(new IdentifierExpression("length"),
                                   Arrays.asList(new Object[] {
                                       new IdentifierExpression("a")
                                   }))
                            .evaluate(ec));
    }

    /**
     * Creates a new UnitTest to execute the specified test.<p>
     *
     * @param name The name of the test method to run.
     */
    public LengthTests(String name)
    {
        super(name);
    }
}
