/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.function;

import junit.framework.Assert;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import net.christopherschultz.evaluator.EvaluationContext;
import net.christopherschultz.evaluator.EvaluationException;
import net.christopherschultz.evaluator.Expression;
import net.christopherschultz.evaluator.parser.ExpressionParser;
import net.christopherschultz.evaluator.parser.ParseException;
import net.christopherschultz.evaluator.util.DefaultEvaluationContext;

public class MathFunctionTests
    extends TestCase
{
    EvaluationContext ec;

    private static final Integer ZERO = new Integer(0);
    private static final Integer ONE = new Integer(1);

    public void setUp()
    {
        ec = new DefaultEvaluationContext();

        ec.set("pow", new MathFunction.pow());
    }

    private Object evaluate(String expression,
                            EvaluationContext ec)
        throws EvaluationException, ParseException
    {
        Expression exp = ExpressionParser.parseExpression(expression);

        return exp.evaluate(ec);
    }

    public void testPow()
        throws Exception
    {
        Assert.assertEquals("pow() does not work",
                            new Double(8), evaluate("pow(2,3)", ec));
    }

    public void testEmptyPow()
        throws Exception
    {
        try
        {
            evaluate("pow()", ec);

            Assert.fail("pow should fail with incorrect arguments");
        }
        catch (EvaluationException ee)
        {
            // Expected result
        }
    }

    public void testMinRegular()
        throws Exception
    {
        MathFunction.min f = new MathFunction.min();

        Object o = f.call(ec, "min", new Object[] { new Double(1.0),
                                                    new Double(1.5) });

        Assert.assertEquals(new Double(1.0), o);
    }

    public void testMinNullOk()
        throws Exception
    {
        ec.setProperty(MathFunction.ALLOW_NULL_COMPARISON, Boolean.TRUE);
        MathFunction.min f = new MathFunction.min();

        Object o = f.call(ec, "min", new Object[] { new Double(1.0),
                                                    null,
                                                    new Double(1.5) });

        Assert.assertEquals(new Double(1.0), o);
    }

    public void testMinNullNo()
        throws Exception
    {
        ec.setProperty(MathFunction.ALLOW_NULL_COMPARISON, Boolean.FALSE);
        MathFunction.min f = new MathFunction.min();

        Object o = f.call(ec, "min", new Object[] { new Double(1.0),
                                                    null,
                                                    new Double(1.5) });

        Assert.assertNull(o);
    }

    public void testMaxRegular()
        throws Exception
    {
        MathFunction.max f = new MathFunction.max();

        Object o = f.call(ec, "max", new Object[] { new Double(1.0),
                                                    new Double(1.5) });

        Assert.assertEquals(new Double(1.5), o);
    }

    public void testMaxNullOk()
        throws Exception
    {
        ec.setProperty(MathFunction.ALLOW_NULL_COMPARISON, Boolean.TRUE);
        MathFunction.max f = new MathFunction.max();

        Object o = f.call(ec, "max", new Object[] { new Double(1.0),
                                                    null,
                                                    new Double(1.5) });

        Assert.assertEquals(new Double(1.5), o);
    }

    public void testMaxNullNo()
        throws Exception
    {
        ec.setProperty(MathFunction.ALLOW_NULL_COMPARISON, Boolean.FALSE);
        MathFunction.max f = new MathFunction.max();

        Object o = f.call(ec, "max", new Object[] { new Double(1.0),
                                                    null,
                                                    new Double(1.5) });

        Assert.assertNull(o);
    }

    public void testMaxNullOnly()
        throws Exception
    {
        MathFunction.max f = new MathFunction.max();

        Object o = f.call(ec, "max", new Object[] { null });

        Assert.assertNull(o);
    }

    public void testMaxNullsOnly()
        throws Exception
    {
        MathFunction.max f = new MathFunction.max();

        Object o = f.call(ec, "max", new Object[] { null, null });

        Assert.assertNull(o);
    }

    public void testFloorFunction()
        throws Exception
    {
        MathFunction.floor f = new MathFunction.floor();

        Object o = f.call(ec, "floor", new Object[] { new Double(1.5) });

        Assert.assertEquals(new Double(1.0), o);
    }

    public void testFloorFunctionNullOk()
        throws Exception
    {
        ec.setProperty(MathFunction.NULL_RETURNS_NULL, Boolean.TRUE);
        MathFunction.floor f = new MathFunction.floor();

        Object o = f.call(ec, "floor", new Object[] { null });

        Assert.assertNull(o);
    }

    public void testFloorFunctionNullNo()
        throws Exception
    {
        ec.setProperty(MathFunction.NULL_RETURNS_NULL, Boolean.FALSE);
        MathFunction.floor f = new MathFunction.floor();

        try
        {
            f.call(ec, "floor", new Object[] { null });

            Assert.fail("Double function (floor) should fail when NULL argument is used.");
        }
        catch (NullPointerException npe)
        {
            // Expected behavior
        }
    }
}
