/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.function;

import junit.framework.Assert;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.util.ArrayList;

import net.christopherschultz.evaluator.*;
import net.christopherschultz.evaluator.util.DefaultEvaluationContext;

/**
 * UnitTest for Summation (function).
 *
 * @author Chris Schultz
 * @version $Revision: 1.2 $ $Date: 2008-06-20 22:58:48 $
 */
public class SummationTests
    extends TestCase
{
    EvaluationContext ec;
    Expression exp;

    public void setUp()
    {
        ec = new DefaultEvaluationContext();

        ec.set("+", new Summation());
    }

    private Number getNumber(double n, Class type)
    {
        if(Double.class == type)
            return new Double(n);
        if(Float.class == type)
            return new Float((float)n);
        if(Long.class == type)
            return new Long((long)n);
        if(Integer.class == type)
            return new Integer((int)n);
        if(Short.class == type)
            return new Short((short)n);
        if(Byte.class == type)
            return new Byte((byte)n);

        throw new IllegalArgumentException();
    }

    public void doTest(Number[] nums, Class expectedResultType)
        throws EvaluationException
    {
        double expectedResult = 0.0;

        ArrayList args = new ArrayList();
        for(int i=0; i<nums.length; ++i)
        {
            expectedResult += nums[i].doubleValue();
            args.add(new ConstantExpression(nums[i]));
        }

        exp = new FunctionCallExpression(new IdentifierExpression("+"),
                                     args);
        Assert.assertEquals("Addition does not work for "
                            + java.util.Arrays.asList(nums),
                            getNumber(expectedResult, expectedResultType),
                            exp.evaluate(ec));
    }

    public void testIdentity()
        throws EvaluationException
    {
        doTest(new Number[] { new Double(1.0) }, Double.class);
        doTest(new Number[] { new Float(1.0) }, Float.class);
        doTest(new Number[] { new Long((long)1) }, Long.class);
        doTest(new Number[] { new Integer((int)1) }, Integer.class);
        doTest(new Number[] { new Short((short)1) }, Short.class);
        doTest(new Number[] { new Byte((byte)1) }, Byte.class);
    }

    public void testAddition()
        throws EvaluationException
    {
        doTest(new Number[] { new Double(1.0),
                              new Double(1.0)
        }, Double.class);
        doTest(new Number[] { new Float(1.0),
                              new Float(1.0)
        }, Float.class);
        doTest(new Number[] { new Long((long)1),
                              new Long((long)1)
        }, Long.class);
        doTest(new Number[] { new Integer((int)1),
                              new Integer((int)1)
        }, Integer.class);
        doTest(new Number[] { new Short((short)1),
                              new Short((short)1)
        }, Short.class);
        doTest(new Number[] { new Byte((byte)1),
                              new Byte((byte)1)
        }, Byte.class);
    }

    public void testMultipleAddition()
        throws EvaluationException
    {
        doTest(new Number[] { new Double((double)1),
                              new Double((double)1),
                              new Double((double)1)
        }, Double.class);
        doTest(new Number[] { new Float((float)1),
                              new Float((float)1),
                              new Float((float)1)
        }, Float.class);
        doTest(new Number[] { new Long((long)1),
                              new Long((long)1),
                              new Long((long)1)
        }, Long.class);
        doTest(new Number[] { new Integer((int)1),
                              new Integer((int)1),
                              new Integer((int)1)
        }, Integer.class);
        doTest(new Number[] { new Short((short)1),
                              new Short((short)1),
                              new Short((short)1)
        }, Short.class);
        doTest(new Number[] { new Byte((byte)1),
                              new Byte((byte)1),
                              new Byte((byte)1)
        }, Byte.class);
    }

    public void testFloatShortAddition()
        throws EvaluationException
    {
        doTest(new Number[] { new Float(3.14f),
                              new Short((short)3) },
               Float.class);
    }

    public void testShortByteByteAddition()
        throws EvaluationException
    {
        doTest(new Number[] { new Short((short)3),
                              new Byte((byte)1),
                              new Byte((byte)1)
        },
               Short.class);
    }

    public void testIntLongIntIntAddition()
        throws EvaluationException
    {
        doTest(new Number[] { new Integer(15),
                              new Long(5),
                              new Integer(2),
                              new Integer(2)
        },
                               Long.class);
    }

    public void testNegativeIntLongIntIntAddition()
        throws EvaluationException
    {
        doTest(new Number[] { new Integer(-15),
                              new Long(5),
                              new Integer(-2),
                              new Integer(2)
        },
                               Long.class);
    }

    public void testNullArgumentAllowed()
        throws EvaluationException
    {
        ec.setProperty(MathFunction.NULL_RETURNS_NULL, Boolean.TRUE);
        ec.set("null", null); // Just in case ;)

        ArrayList args = new ArrayList();
        args.add(new ConstantExpression(new Integer(1)));
        args.add(new IdentifierExpression("null"));

        exp = new FunctionCallExpression(new IdentifierExpression("+"), args);

        Assert.assertNull("Null argument does not return null result.",
                          exp.evaluate(ec));
    }

    public void testNullArgumentNotAllowed()
        throws EvaluationException
    {
        ec.setProperty(MathFunction.NULL_RETURNS_NULL, Boolean.FALSE);
        ec.set("null", null); // Just in case ;)

        ArrayList args = new ArrayList();
        args.add(new ConstantExpression(new Integer(1)));
        args.add(new IdentifierExpression("null"));

        exp = new FunctionCallExpression(new IdentifierExpression("+"), args);

        try
        {
            exp.evaluate(ec);
            Assert.fail("Null argument does not fail.");
        }
        catch (EvaluationException ee)
        {
            // Expected behavior
        }
    }

    public SummationTests(String name)
    {
        super(name);
    }
}
