/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.util;

import junit.framework.Assert;
import junit.framework.TestCase;

import java.util.Map;

import net.christopherschultz.evaluator.EvaluationContext;
import net.christopherschultz.evaluator.Function;

/**
 * UnitTest for the ChainedEvaluationContext.
 *
 * @author Chris Schultz
 * @version $Revision: 1.3 $ $Date: 2008-06-20 22:58:48 $
 */
public class ChainedEvaluationContextTests
    extends TestCase
{
    EvaluationContext ec;
    ChainedEvaluationContext cet;

    public void setUp()
    {
        ec = new DefaultEvaluationContext();
        cet = new ChainedEvaluationContext(ec);
    }

    class DummyFunction
        implements Function
    {
        public Object call(EvaluationContext ec,
                           String functionName,
                           java.util.List args)
        {
            return null;
        }
    }

    public void testFunctionRegistration()
    {
        Function dummy = new DummyFunction();

        cet.set("dummy", dummy);

        Assert.assertEquals("ChainedEvaluationContext incorrectly registers functions",
                            dummy, cet.get("dummy"));

        Assert.assertNull("ChainedEvaluationContext incorrectly registers functions",
                          ec.get("dummy"));
    }

    public void testFunctionHiding()
    {
        Function dummy = new DummyFunction();
        Function dummy2 = new DummyFunction();

        assertNotEquals("Functions are the same?!", dummy, dummy2);

        ec.set("dummy", dummy);

        Assert.assertEquals("ChainedEvaluationContext incorrectly registers functions",
                            dummy, cet.get("dummy"));

        cet.set("dummy", dummy2);

        Assert.assertEquals("ChainedEvaluationContext incorrectly registers functions",
                            dummy2, cet.get("dummy"));

        cet.set("another_dummy", dummy);
        Assert.assertEquals("ChainedEvaluationContext incorrectly registers functions",
                            dummy, cet.get("another_dummy"));

        ec.set("another-dummy", dummy2);
        Assert.assertEquals("ChainedEvaluationContext incorrectly resolves functions",
                            dummy, cet.get("another_dummy"));
    }

    public void testVariables()
    {

        cet.set("dummy", "dummy");

        Assert.assertEquals("ChainedEvaluationContext incorrectly sets vars",
                            "dummy", cet.get("dummy"));

        Assert.assertNull("ChainedEvaluationContext incorrectly sets vars",
                          ec.get("dummy"));
    }

    public void testVariableHiding()
    {
        ec.set("dummy", "dummy");

        Assert.assertEquals("ChainedEvaluationContext incorrectly sets vars",
                            "dummy", cet.get("dummy"));
        Assert.assertEquals("ChainedEvaluationContext incorrectly sets vars",
                            "dummy", ec.get("dummy"));

        cet.set("dummy", "no dummy");

        Assert.assertEquals("ChainedEvaluationContext incorrectly sets vars",
                            "no dummy", cet.get("dummy"));
        Assert.assertEquals("ChainedEvaluationContext incorrectly sets vars",
                            "dummy", ec.get("dummy"));

        cet.remove("dummy");

        Assert.assertEquals("ChainedEvaluationContext incorrectly removed vars",
                            "dummy", cet.get("dummy"));
        Assert.assertEquals("ChainedEvaluationContext incorrectly removed vars",
                            "dummy", ec.get("dummy"));
    }

    public void testGetAllBindings()
    {
        Assert.assertEquals("", 0, ec.getAll().size());
        Assert.assertEquals("", 0, cet.getAll().size());

        cet.set("one", new Integer(1));
        cet.set("two", new Integer(2));

        Map bindings = cet.getAll();

        Assert.assertEquals("getAll does not work", 2, bindings.size());
        Assert.assertEquals("getAll loses bindings",
                            new Integer(1), bindings.get("one"));
        Assert.assertEquals("getAll loses bindings",
                            new Integer(2), bindings.get("two"));
    }

    public void testGetAllChainedBindings()
    {
        Assert.assertEquals("DefaultEvaluationContext is not empty",
                            0, ec.getAll().size());
        Assert.assertEquals("ChainedEvaluationContext is not empty",
                            0, cet.getAll().size());

        ec.set("one", new Integer(1));
        ec.set("two", new Integer(2));

        Map bindings = cet.getAll();

        Assert.assertEquals("getAll() does not work", 2, bindings.size());
        Assert.assertEquals("", new Integer(1), bindings.get("one"));
        Assert.assertEquals("", new Integer(2), bindings.get("two"));

        cet.set("three", new Integer(3));

        bindings = cet.getAll();
        Assert.assertEquals("cec.getAll() does not work", 3, bindings.size());
        Assert.assertEquals("getAll loses bindings",
                            new Integer(1), bindings.get("one"));
        Assert.assertEquals("getAll loses bindings",
                            new Integer(2), bindings.get("two"));
        Assert.assertEquals("getAll loses bindings",
                            new Integer(3), bindings.get("three"));
    }

    public void assertNotEquals(String message,
                                Object expected,
                                Object actual)
    {
        Assert.assertTrue(message,
                          !(expected.equals(actual)));
    }

    /**
     * Creates a new UnitTest to execute the specified test.<p>
     *
     * @param name The name of the test method to run.
     */
    public ChainedEvaluationContextTests(String name)
    {
        super(name);
    }
}
