/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.util;

import junit.framework.Assert;
import junit.framework.TestCase;

import java.util.Map;
import java.util.HashMap;

import net.christopherschultz.evaluator.EvaluationContext;
import net.christopherschultz.evaluator.function.MathFunction;

/**
 * Unit tests for NegatedExpression.<p>
 *
 * @author Chris Schultz
 * @version $Revision: 1.4 $ $Date: 2008-06-20 22:58:48 $
 */
public class DefaultEvaluationContextTests
    extends TestCase
{
    DefaultEvaluationContext ec;

    public void testJavaFieldConstantLoading()
        throws Exception
    {
        HashMap constants = new HashMap();

        constants.put("constant.null", "null");
        constants.put("constant.true", "java.lang.Boolean#TRUE");
        constants.put("constant.false", "java.lang.Boolean#FALSE");

        ec.load(constants);

        Assert.assertNull("Null doesn't load properly", ec.get("null"));
        Assert.assertEquals("Booleans don't load properly",
                            Boolean.TRUE, ec.get("true"));
        Assert.assertEquals("Booleans don't load properly",
                            Boolean.FALSE, ec.get("false"));
    }

    public void testNumericConstantLoading()
        throws Exception
    {
        HashMap constants = new HashMap();

        constants.put("constant.ONE", "java.lang.Integer(1)");
        constants.put("constant.TWO", "java.lang.Long(2)");
        constants.put("constant.THREE", "java.lang.Float(3)");
        constants.put("constant.FOUR", "java.lang.Double(4)");
        constants.put("constant.FIVE", "java.lang.Byte(5)");
        constants.put("constant.SIX", "java.lang.Short(6)");
        constants.put("constant.SEVEN", "java.lang.String(7)");

        ec.load(constants);

        Assert.assertEquals("Integers don't load properly",
                            new Integer(1), ec.get("ONE"));
        Assert.assertEquals("Longs don't load properly",
                            new Long(2), ec.get("TWO"));
        Assert.assertEquals("Floats don't load properly",
                            new Float(3), ec.get("THREE"));
        Assert.assertEquals("Doubles don't load properly",
                            new Double(4), ec.get("FOUR"));
        Assert.assertEquals("Bytes don't load properly",
                            new Byte((byte)5), ec.get("FIVE"));
        Assert.assertEquals("Shorts don't load properly",
                            new Short((short)6), ec.get("SIX"));
        Assert.assertEquals("Strings don't load properly",
                            "7", ec.get("SEVEN"));
    }

    public void testPropertyLoading()
        throws Exception
    {
        HashMap constants = new HashMap();

        constants.put("property.net.christopherschultz.evaluator.function.MathFunction#ALLOW_NULL_COMPARISON", "java.lang.Boolean#TRUE");

        ec.load(constants);

        Assert.assertEquals("Properties don't load properly",
                            Boolean.TRUE, ec.getProperty(MathFunction.ALLOW_NULL_COMPARISON));
    }

    public void testGetAllBindings()
    {
        Assert.assertEquals("", 0, ec.getAll().size());

        ec.set("one", new Integer(1));
        ec.set("two", new Integer(2));

        Map bindings = ec.getAll();

        Assert.assertEquals("", 2, bindings.size());
        Assert.assertEquals("", new Integer(1), bindings.get("one"));
        Assert.assertEquals("", new Integer(2), bindings.get("two"));
    }

    public void setUp()
    {
        ec = new DefaultEvaluationContext();
    }

    /**
     * Creates a new UnitTest to execute the specified test.<p>
     *
     * @param name The name of the test method to run.
     */
    public DefaultEvaluationContextTests(String name)
    {
        super(name);
    }
}
